# -*- coding: utf-8 -*-
from lxml.html import fromstring
from lxml import etree
import sys


class DemoExtractor(object):
    """

    """

    def __init__(self):
        super(DemoExtractor, self).__init__()

    def extractor(self, text):
        """
        将一个页面文本解析为结构化信息的字典
        Args:
            text: 需要解析的文本
        Returns:
            数组: 每条为一个完整记录，记录由字典格式保存
        """
        results = list()
        try:
            tree = fromstring(text.decode("utf-8", "ignore"))  # 这种方式 可使用cssselect  不然linux 不能使用
            # soup = BeautifulSoup(text.decode('utf-8', 'ignore'), 'lxml')
            # navs_div = soup.find('div', id='navs')
            # if navs_div:
            #     depart_divs = navs_div.find_all('h2', recursive=True)
            #     if len(depart_divs) > 0:
            #         for depart in depart_divs:
            #             url = depart.find('a', recursive=True)['href']
            #             department_id = url.split('_')[2]
            #             results.append(
            #                 {
            #                     'name': str(depart.get_text()),
            #                     'department_id':department_id
            #                 })
        except Exception:
            pass
        return results



from platform import system as system_name  # Returns the system/OS name
from os import system as system_call  # Execute a shell command


# 下载中心redis
DOWNLOADER_CENTER_REDIS = {
    'outer': {
        'host': '115.159.3.51',
        'port': 53189,
        'db': 0,
        'password': '&redisd0o#2@1951'
    },
    'inner': {
        'host': '10.154.199.106',
        'port': 53189,
        'db': 0,
        'password': '&redisd0o#2@1951'
    }
}


def ping(ip):
    try:
        parameters = "-n 1" if system_name().lower() == "windows" else "-c 1"
        return system_call("ping " + parameters + " " + ip + " > log.txt") == 0
    except:
        return False

if ping(DOWNLOADER_CENTER_REDIS['inner']['host']):
    ENVIR = 'inner'
else:
    ENVIR = 'outer'



# -*- coding: utf8 -*-
import os
import re
import sys



class SpiderDownloader(object):
    """
    下载器的抽象类
    用于下载指定url的网页内容
    下载器可能自身具备下载功能，也可能是作为与下载中心交互的接口
    """

    def __init__(self):
        pass

    def set(self, headers, config={}, urls=[]):
        """
        设置http头或者将相关配置发到下载中心
        Args:
            headers: 字典，http头信息，
                举例，{'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64; rv:47.0) Gecko/20100101 Firefox/47.0'}
            config: 字典，可为空，用于向下载中心发送下载类型、优先级、抓取频率等相关信息，
                或者向下载函数设置超时等相关信息，
                举例：{'type':0, 'priority':5, 'frequency':1} 或 {'timeout':10}
            urls: 数组，不使用下载中心则为空，用于将待下载的url预先发到下载中心进行下载
        Returns:
            1: 正常, 0: 出错
        """
        return 0

    def get(self, url):
        """
        下载特定url的页面或者向下载中心请求特定url的结果
        Args:
            url: 要获取结果的一个url
        Returns :
            None: 未取到结果，可能是下载中心还没有下载完成
            文本: 下载结果
        """
        return None


class SpiderExtractor(object):
    """
    解析器的抽象类
    用于将特定网页解析成结构化信息
    每个页面有一个独立的SpiderExtractor类
    """

    def __init__(self):
        pass

    def extractor(self, text):
        """
        将一个页面文本解析为结构化信息的字典
        Args:
            text: 需要解析的文本
        Returns:
            数组: 每条为一个完整记录，记录由字典格式保存
        """
        return []


class SpiderStore(object):
    """
    存储器的抽象类
    用于将结构化信息保存到指定存储媒介中
    Attributes:
        fields: 用于保持抽取器字典的key与数据库字段的对应关系，
            如果结果字典的某个key不包含在fields中，则将直接使用key作为字段名
    """

    def __init__(self):
        self.fields = {}
        pass

    def store(self, results, type=1, keys=[]):
        """
        将一个数组存储到指定的存储媒介中
        Args:
            reuslts: 数组，每条为一个完整记录，记录由字典格式保存
            type: 1-只插入（出错则忽略），2-只更新（原记录不存在则忽略），3-插入更新（无记录则插入，有记录则更新）
            keys: 只插入则为空，更新则存储关键字典，此字段将作为更新的where条件
        Returns:
            1: 正常, 0: 出错
        """
        pass


class Spider(object):
    """
    爬虫的抽象类
    Attributes:
        headers: http头，同SpiderDownloader
        config: 抓取配置，同SpiderDownloader
        urls: 待抓取url，同SpiderDownloader
        downloader: 下载器，通常一个Spider配一个下载器
        extractor: 解析器，一个Spider可以配多个解析器
        store: 存储器，通常一个Spider配一个存储器
    """

    def __init__(self):
        self.headers = {}
        self.config = {}
        self.urls = []
        self.downloader = SpiderDownloader()
        self.extractor = SpiderExtractor()
        self.store = SpiderStore()
        pass

    def get_seeds(self):
        """
        获取urls
        """
        f = open('seeds.txt')
        for line in f:
            self.urls.append(line[:-1])
        f.close()

    def prepare(self):
        """
        用于对接下载中心的异步抓取，作为独立操作运行，投递任务后即退出
        """
        self.get_seeds()
        self.downloader.set(self.headers, self.config, self.urls)

    def run(self):
        """
        运行主函数，可独立运行（独立下载器模式），也可放在prepare之后运行（下载中心模式）
        """
        self.get_seeds()
        for url in self.urls:
            content = self.downloader.get(url)
            results = self.extractor.extractor(content)
            if len(results) > 0:
                self.store.store(results)

    def test(self, url):
        """
        测试单体url的结果
        """
        content = self.downloader.get(url)
        results = self.extractor.extractor(content)
        print(results)


"""
def Main():
    s = Spider()
    #s.prepare
    s.run()

if __name__ == "__main__":
    Main();
"""



# -*- coding: utf8 -*-

import redis
import time
from download_center.store import config


class baiduCookie(object):
    def __init__(self):
        self.redisPool = redis.ConnectionPool(**(config.DOWNLOADER_CENTER_REDIS[config.ENVIR]))
        self.redisClient = redis.StrictRedis(connection_pool=self.redisPool)
        self.pc_cookie_key = 'baidu:pc:cookies'
        self.mb_cookie_key = 'baidu:mb:cookies'

    def getCookies(self, num=0, key_name='pc'):
        '''
        [获取一个有效的cookie 正序]
        :param num:
        :param key_name:
        :return:
        '''
        # noinspection PyBroadException
        try:
            if key_name == 'pc':
                return self.redisClient.zrange(self.pc_cookie_key, 0, num)
            elif key_name == 'mb':
                return self.redisClient.zrange(self.mb_cookie_key, 0, num)
            else:
                return []
        except Exception:
            return []

    def getLastCookies(self, num=0, key_name='pc'):
        '''
         [获取最新的cooie]
        :param num:
        :param key_name:
        :return:
        '''
        # noinspection PyBroadException
        try:
            if key_name == 'pc':
                return self.redisClient.zrevrange(self.pc_cookie_key, 0, num)
            elif key_name == 'mb':
                return self.redisClient.zrevrange(self.mb_cookie_key, 0, num)
            else:
                return []
        except Exception:
            return []

    def addCookies(self, cookie, key_name='pc'):
        '''
        [添加一个值]
        :param cookie:
        :param key_name:
        :return:
        '''
        # noinspection PyBroadException
        try:
            score = int(round(time.time() * 1000))
            if key_name == 'pc':
                return self.redisClient.zadd(self.pc_cookie_key, {cookie: score})
            elif key_name == 'mb':
                return self.redisClient.zadd(self.mb_cookie_key, {cookie: score})
            else:
                return 0
        except Exception:
            return 0

    def delCookies(self, cookie, key_name='pc'):
        '''
        删除cookie
        :param cookie:
        :return:
        '''
        # noinspection PyBroadException
        try:
            if key_name == 'pc':
                return self.redisClient.zrem(self.pc_cookie_key, cookie)
            elif key_name == 'mb':
                return self.redisClient.zrem(self.mb_cookie_key, cookie)
            else:
                return 0
        except Exception:
            return 0

    def totalCookies(self, key_name='pc'):
        '''
        [统计个数]
        :return:
        '''
        # noinspection PyBroadException
        try:
            if key_name == 'pc':
                return self.redisClient.zcard(self.pc_cookie_key)
            elif key_name == 'mb':
                return self.redisClient.zcard(self.mb_cookie_key)
            else:
                return 0
        except Exception:
            return 0


if __name__ == '__main__':
    bc = baiduCookie()
    print(bc.getCookies(1, 'p'))
    print(bc.totalCookies('mb'))



# -*- coding: utf-8 -*-
"""
 @Time: 2019/6/10 13:21
"""
import time
import os
import sys
import traceback
import hashlib
import urllib
import pymysql
import json
import time
import base64
from pymysql.converters import escape_string

from datetime import datetime

import re
import urllib3
urllib3.disable_warnings()
import requests
requests.packages.urllib3.disable_warnings()

def util_md5(s):
    m = hashlib.md5()
    m.update(s.encode(encoding='gbk'))
    return m.hexdigest()


class HtmlLocalDownloader(object):
    """
    html下载器
    """

    def __init__(self, set_mode='db', get_mode='db', store_type=5):  # 兼容 保留
        self.user_id = 0
        self.REQUEST_TIMEOUT = 20

    def downloader_set_param(self, request, store_type=5):
        url_type = None
        new_urls = list()
        for url in request.urls:
            if url_type is None:
                url_type = url['type']
            new_url = dict()
            new_url['url'] = url['url']
            if 'unique_key' in url.keys():
                md5 = util_md5(escape_string(url['url']) + str(url['unique_key']))
            else:
                md5 = util_md5(escape_string(url['url']))
            new_url['md5'] = md5
            new_urls.append(new_url)

            url['unique_md5'] = md5
        if len(new_urls) > 0:
            params = {
                'user_id': self.user_id,
                'url_type': url_type,
                'header': request.headers,
                'redirect': 0,
                'priority': 2,
                'single': 0,
                'store_type': store_type,
                'urls': new_urls,
                'concurrent_num': 0,  # 默认
                'conf_district_id': 0
            }
            if 'redirect' in request.config and request.config['redirect'] == 1:
                params['redirect'] = request.config['redirect']
            if 'post_data' in request.config:
                params['post_data'] = request.config['post_data']
            if 'filter' in request.config:
                params['filter'] = request.config['filter']
            return params
        return None

    def set(self, request):
        try:
            results = dict()
            param = self.downloader_set_param(request)
            for url in param['urls']:
                results[url['md5']] = 1
            return results
        except Exception:
            print(traceback.format_exc())
            return 0

    @staticmethod
    def encoding(data):
        types = ['utf-8', 'gb2312', 'gbk', 'gb18030', 'iso-8859-1']
        for t in types:
            try:
                return data.decode(t)
            except Exception:
                pass
        return None

    def get(self, request):
        param = self.downloader_set_param(request)
        if param is None:
            return 0
        urls = param['urls']
        if len(urls) > 0:
            try:
                results = dict()
                url_type = param.get("url_type", 1)

                headers = param.get("header", {})
                data = param.get("post_data", None)
                method = param.get("method", None)
                filter = param.get('filter', 0)

                for url in urls:
                    result = {"url": url["url"], "status": "3", "result": "", "header": "",
                              "redirect_url": "", "code": 0, "type": url_type}

                    # start = time.time()
                    self.download_html_by_requests(url["url"], result, data, headers, req=requests, filter=filter,
                                                   method=method)
                    # print(
                    #     "url: {}; length: {}; time: {}".format(url["url"], len(result["result"]), time.time() - start))

                    results[url['md5']] = result
                return results
            except Exception:
                print(traceback.format_exc())
        return 0

    def download_html_by_requests(self, url, result, data, headers, req=None, filter=0, method=None):
        """
        :param task:
        :param result:
        :param data:
        :param headers:
        :param req:        requests
        :param filter:  0 默认不过滤 1 过滤
        :return:
        """
        try:
            response = self.transfer_requests(req, url, data, headers, None, method)
            try:
                if response.encoding == 'ISO-8859-1':
                    encodings = requests.utils.get_encodings_from_content(response.text)
                    if encodings:
                        encoding = encodings[0]
                    else:
                        encoding = response.apparent_encoding
                else:
                    encoding = response.encoding
            except:
                encoding = "utf-8"

            result["code"] = 0 if response.status_code == 200 else response.status_code
            if url != response.url:
                result["redirect_url"] = response.url

            result["status"] = 2
            encode_content = response.content.decode(encoding, 'replace').encode('utf-8', 'replace')
            if filter == 1:
                encode_content = self.filter_html(encode_content)

            if not isinstance(encode_content, str):
                encode_content = encode_content.decode(encoding="utf-8", errors='ignore')  # bytes to str
            result["result"] = encode_content
            if str(result["type"]) == "2":
                result["header"] = str(self.get_requests_cookie(response))
        except requests.exceptions.ConnectTimeout:
            # 代理问题 连接超时
            print(traceback.format_exc())
        except Exception:
            # url 请求 问题
            print(traceback.format_exc())

    @staticmethod
    def get_requests_cookie(res):
        cookies = requests.utils.dict_from_cookiejar(res.cookies)
        cookies_str = ""
        for k in cookies:
            cookies_str += ";{}={}".format(k, cookies[k])
        return cookies_str[1: -1]

    def transfer_requests(self, req, url, data, headers, proxies, method):
        if proxies:
            if method:
                if method == "get":
                    response = req.get(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                       headers=headers, verify=False)
                else:
                    response = req.post(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                        data=data, headers=headers, verify=False)
            else:
                if data:
                    response = req.post(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                        data=data, headers=headers, verify=False)
                else:
                    response = req.get(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                       headers=headers, verify=False)
        else:
            if method:
                if method == "get":
                    response = req.get(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                       headers=headers, verify=False)
                else:
                    response = req.post(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                        data=data, headers=headers, verify=False)
            else:
                if data:
                    response = req.post(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                        data=data, headers=headers, verify=False)
                else:
                    response = req.get(url=url, timeout=self.REQUEST_TIMEOUT, proxies=proxies,
                                       headers=headers, verify=False)
        return response

    @staticmethod
    def filter_html(content):
        """
        过滤html 无用字符串
        备注
            1、 去除换行
            2、 去除<script
            3、 去<style
            4、 多空格 变成一个
            5、 多tab 键 变成一个
            6、 去<link

            --
            去备注  <!--  -->
            去 style none
        :param content:
        :return:
        """
        if not isinstance(content, str):
            content = content.decode(encoding="utf-8", errors='ignore')  # bytes to str
        content = re.sub('\\n', ' ', content)
        tlist = re.findall(r'<script(.*?)</script>', content)
        for t in tlist:
            content = content.replace("<script" + t + "</script>", "")
        tlist = re.findall(r'<style(.*?)</style>', content)
        for t in tlist:
            content = content.replace("<style" + t + "</style>", "")
        tlist = re.findall(r'<link(.*?)>', content)
        for t in tlist:
            content = content.replace("<link" + t + ">", "")
        content = re.sub(' +', ' ', content)
        content = re.sub('\\t+', ' ', content)
        content = re.sub('\\r+', ' ', content)
        return content




# -*- coding: utf-8 -*-
"""
 @Time: 2019/5/28 13:47
"""

import time
import os
import sys

text = "家开发部"
print(type(text))

# text = text.encode(encoding="utf-8", errors='strict')
text = text.encode(encoding="utf-8", errors='ignore')
# text = text.encode(encoding="utf-8",)
print(type(text))
print(isinstance(text, bytes))


text = text.decode(encoding="utf-8")
print(type(text))

print(text)

print(isinstance(text, str))
print(isinstance(text, bytes))


# -*- coding: utf8 -*-

import time
import sys


from pymongo import *

class StoreMongo(object):

    """
    mongoDB数据库链接
    mongodb://{username}:{password}@{host}:{port}/{db_name}?authMechanism=MONGODB-CR
    Args:
        host:数据库ip
        user:数据库用户名
        password:数据库用户密码
        db:数据库名
        port:数据库端口，默认3306
        authset:数据库认证模式，mongodb2.+默认为MONGODB-CR；3.0+默认为SCRAM-SHA-1
    """
    def __init__(self, host, port, user, password, db, authset='MONGODB-CR'):
        uri = 'mongodb://%s:%s@%s:%d/%s?authMechanism=%s' % (user, password, host, port, db, authset)
        client = MongoClient(uri)
        self.db = client[db]



# -*- coding: utf8 -*-
from .downloader import  Downloader
import sys

class HtmlRender(Downloader):
    """
    html渲染器
    """

    def __init__(self, set_mode='db', get_mode='db'):
        super(HtmlRender, self).__init__(set_mode, get_mode)



#!/usr/bin/python
# -*- coding: utf8 -*-

from platform import system as system_name  # Returns the system/OS name
from os import system as system_call  # Execute a shell command
import sys


class Ping(object):

    @staticmethod
    def ping(ip):
        try:
            parameters = "-n 1" if system_name().lower() == "windows" else "-c 1"
            # return system_call("ping " + parameters + " " + ip) == 0
            return system_call("ping " + parameters + " " + ip + " > log.txt") == 0
        except:
            return False



#!/usr/bin/python
# -*- coding: utf8 -*-
import os
import sys
import time
import logging


class UtilLogger(object):
    """
    建立日志文件，并以特定格式输出日志
    Args:
        name:logger名字
        logfile_name 日志文件名
        level:调试级别，日志中只打印高于此级别的日志，例如logging.DEBUG、logging.info，此级别可以在set_level函数里设置
    """

    def __init__(self, name, logfile_name=None, level=logging.DEBUG):
        self.logger = logging.getLogger(name)
        self.logger.setLevel(level)
        formatter = logging.Formatter("%(asctime)s [%(levelname)s] %(name)s - %(message)s")
        ch = None
        if logfile_name is None:
            ch = logging.StreamHandler()
        else:
            logDir = os.path.dirname(logfile_name)
            if logDir != "" and not os.path.exists(logDir):
                os.mkdir(logDir)
                pass
            now = time.localtime()
            suffix = '.%d%02d%02d' % (now.tm_year, now.tm_mon, now.tm_mday)
            ch = logging.FileHandler(logfile_name + suffix)
        ch.setLevel(logging.DEBUG)
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)

    def set_level(self, level):
        """
        设置调试等级
        Args:
            level，字符串，可选debug、info、warning、error
        """
        if level.lower() == "debug":
            self.logger.setLevel(logging.DEBUG)
        elif level.lower() == "info":
            self.logger.setLevel(logging.INFO)
        elif level.lower() == "warning":
            self.logger.setLevel(logging.WARNING)
        elif level.lower() == "error":
            self.logger.setLevel(logging.ERROR)

    def debug(self, message):
        '''
        打印函数，最低调试级别的打印，
        Args:
            message为要打印的信息
        info/warn/error函数与此类似
        '''
        self.logger.debug(message)

    def info(self, message):
        self.logger.info(message)

    def warn(self, message):
        self.logger.warn(message)

    def error(self, message):
        self.logger.error(message)


'''
def test():
    log = UtilLogger('testname','test')
    log.set_level('info')
    log.debug('++++++++++++++')
    log.info('--------------')
    log.warn('==============')
    log.error('_____________')
if __name__ == '__main__':
    test()
'''



# -*- coding: utf-8 -*-
"""
 @Time: 2019/5/28 11:17
"""

import time
import os
import sys

import traceback
#
# print('获取初始请求出错: {}'.format(123231))
#
# print(36267327)

# time.sleep(5)
try:
    # print("test")
    da = "dsds"
    int(da)
except Exception:
    # print('获取初始请求出错: {}'.format(traceback.format_exc()))
    print(traceback.format_exc())
    # traceback.print_exc()


# -*- coding: utf8 -*-
# from util.util_ping import Ping
from platform import system as system_name  # Returns the system/OS name
from os import system as system_call  # Execute a shell command


def ping(ip):
    try:
        parameters = "-n 1" if system_name().lower() == "windows" else "-c 1"
        return system_call("ping " + parameters + " " + ip + " > log.txt") == 0
    except:
        return False

# request timeout
REQUEST_TIMEOUT = 30

# IP_HOST = '127.0.0.1:9090'                       # 测试地址
IP_HOST = '182.254.155.218:9090'                   # 上線地址
# IP_HOST = '182.254.155.218:9090'                   # 上線地址

# validate_accout
VALIDATE_ACCOUNT_URL = "http://{}/download/login".format(IP_HOST)

ADD_BLACK_IP = "http://{}/download/addBlackIp"         # 添加黑名单

# sendTask
DOWNLOADER_SENDTASK = "http://{}/download/setTask"

# getResult
DOWNLOADER_GETRESULT = "http://{}/download/getResult"

TASK_SCHEDULER_IP = "http://{}/adslGetIp"

REQUEST_RETYR_SLEEP = 2     # 请求异常等待时间

REQUEST_RETYR_MAX_SLEEP = 10    # 请求异常 超過幾次 等待时间

# 内网ip
DOWNLOADER_CENTER_INNER_IP = "10.105.72.2"

DOWNLOADER_CENTER_IP = {
    "inner": "10.105.72.2",
    "outer": "182.254.155.218"
}

if ping(DOWNLOADER_CENTER_INNER_IP):
    ENVIR = 'inner'
else:
    ENVIR = 'outer'

downloader_ip = None


# -*- coding: utf-8 -*-
import hashlib
import json
import os
import string
import sys
import random
import base64
import time
import traceback
import uuid
import urllib
import urllib.parse
from http import cookiejar

from urllib.request import Request, HTTPCookieProcessor, build_opener

from scrapy.http.cookies import CookieJar

from demo.extractor.demo_extractor import DemoExtractor

from download_center.new_spider.downloader.downloader import SpiderRequest
from download_center.new_spider.spider.basespider import BaseSpider
from download_center.util.util_log import UtilLogger

# 线上测试
PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
# print("PROJECT_PATH", PROJECT_PATH)
sys.path.append(PROJECT_PATH)
# sys.path.append(os.path.join(PROJECT_PATH, 'demo'))
from demo.extractor.baidu_extractor import BaiduExtractor
# from extractor.baidu_extractor import BaiduExtractor


class DemoSpider(BaseSpider):
    def __init__(self, remote=True):
        super(DemoSpider, self).__init__(remote=remote)
        self.log = UtilLogger('DemoSpider', os.path.join(os.path.dirname(os.path.abspath(__file__)), 'log_demo_spider'))
        self.ext = BaiduExtractor()

    def get_user_password(self):
        return 'zhangbz', 'Welcome#1'

    def tn(self):
        return random.choice(
            ['50000021_hao_pg', 'site888_3_pg', '77021190_cpr', 'site5566', '520com_pg', '51010079_cpr'])

    # 'url': 'https://www.baidu.com/s?%s' % (urllib.parse.urlencode({'word': '联想', 'ie': 'utf-8', 'tn': self.tn()})),
    def start_requests(self):
        try:
           for i in range(1000):
               urls = [{
                   'url': 'https://m.baidu.com/s?%s' % (urllib.parse.urlencode({'word': '联想','pn':10})),
                   'type': 1,
                   'unique_key':self.get_unique_key()#默认md5 url 字段，不写则表示相同链接只查一次
               }]
               headers = {
                   "User-Agent": "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.71 Mobile Safari/537.36",
                   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
                   # 'Accept-Language':'zh-CN,zh;q=0.9,en-CN;q=0.8,en;q=0.7,ar-XB;q=0.6,ar;q=0.5',
                   'Cookie': self.get_baidu_mb_cookie()
               }
               config={"param": {'cu':'https://www.baidu.com'},"conf_district_id":3}
               request = SpiderRequest(headers=headers, urls=urls,config=config)
               self.sending_queue.put(request)
               time.sleep(0.1)

        except Exception:
            self.log.error('获取初始请求出错: {}'.format(traceback.format_exc()))


    #获取百度移动端cookie
    def get_baidu_mb_cookie(self):
        cookies = None
        try:
            cookie_jar = cookiejar.CookieJar()
            request = Request('https://m.baidu.com/tc?tcreq4log=1', headers={})
            handlers = [HTTPCookieProcessor(cookie_jar)]
            opener = build_opener(*handlers)
            opener.open(request, timeout=10)
            for cookie in cookie_jar:
                if cookie.name == 'BDORZ':
                    cookies = 'BDORZ=' + cookie.value
                    break
        except Exception:
            pass
        return cookies

    #获取百度pc 端cookie
    def get_baidu_pc_cookie(self):
        letters_one = []
        letters_two = []
        for _ in range(32):
            letters_one.append(random.choice(string.ascii_uppercase + string.digits))

        for _ in range(27):
            letters_two.append(random.choice(string.lowercase + string.digits))

        return 'BAIDUID=%s:FG=1' % (''.join(letters_one)) + '; ' + 'BA_HECTOR=%s' % (''.join(letters_two))

    def get_stores(self):
        # 存储器
        stores = list()
        return stores

    def deal_response_results_status(self, task_status, url, result, request):
        """
        Args:
            task_status:
            url:
            result:
            request:

        Returns:
        根据自己的解析类型做不同的处理，默认返回html
        """
        if task_status == '2':
            config = request.config
            try:
                # result = json.loads(result["result"])
                if '联想' not in result["result"]:
                    print("结果抓取失败")
                else:
                    print("成功!，长度：{},url：{}".format(len(result), url['url']))
            except Exception as e:
                print('失败！url：{}'.format( url['url']))

            # rdata = self.ext.ext ractor(result["result"])
            # self.store_queue.put(result)
            # if isinstance(rdata, int):
            #     print("request ua: {}".format(request.headers["User-Agent"]))
            # else:
            #     result_d, include, keyword_list = rdata
            #     print(result_d)
            #     print(include)
            #     print(keyword_list)
            #
            # with open("html_py3_3.txt", 'w', encoding="utf-8") as f:
            #     f.write(result["result"])
        else:
            self.log.info('抓取失败: {}'.format(url))

    def to_store_results(self,  results, stores):
        """
        结果存储按需使用
        :return:
        """
        pass


def main():
    spider = DemoSpider(remote=True)
    spider.run(1, 1, 1, 1, record_log=True)   # 测试
    # spider.run(spider_count=1000, record_log=True)
    # spider.run(record_log=True)               # error



if __name__ == '__main__':
    main()



# -*- coding: utf8 -*-
import pymysql
import time
from DBUtils.PooledDB import PooledDB
from DBUtils.PersistentDB import PersistentDB

from pymysql.converters import escape_string

import traceback
from datetime import datetime


class StoreMysqlPool(object):
    """
    mysql读写相关操作，pymysql，使用PooledDB连接池
    Args:
        host:数据库ip
        user:数据库用户名
        password:数据库用户密码
        db:数据库名
        port:数据库端口，默认3306
        mincached:最少的空闲连接数， 默认为1
        charset:数据库编码，默认utf8
    """

    def __init__(self, host="", user="", password="", db="", port=3306, mincached=1,
                 pattern=1, charset="utf8"):
        self.host = host

        if pattern == 1:
            self.pool = PooledDB(pymysql, mincached, host=host, user=user, passwd=password, db=db, port=port,
                                 charset=charset)
            self.close_able = True
        elif pattern == 2:
            self.pool = PersistentDB(pymysql, host=host, user=user, passwd=password, db=db, port=port,
                                     charset=charset)

            self.close_able = False
        self._last_use_time = time.time()

    def close(self, db):
        if db is not None:
            db.close()

    def connect(self):
        return self.pool.connection()

    @staticmethod
    def _cursor(db):
        return db.cursor()

    def query(self, sql):
        """
        根据sql查询
        Returns：
            数组的数组，外层数组元素为一行，内存数组元素为一行的一列
        """
        db = self.connect()
        cur = self._cursor(db)
        rows = []
        try:
            cur.execute(sql)
            db.commit()
            rows = cur.fetchall()
        except pymysql.OperationalError:
            print(traceback.format_exc())
        except Exception:
            print(traceback.format_exc())
        finally:
            cur.close()
            self.close(db)
        return rows

    def count(self, tb):
        """
        返回某表的行数
        Args:
            tb:字符串，表名称
        """
        sql = 'select count(*) from %s' % tb
        results = self.query(sql)
        if len(results) == 1 and len(results[0]) == 1:
            return int(results[0][0])

    def do(self, sql, flag='lastrowid'):
        """
        执行sql，insert/delete/update操作
        Args:
            sql:要执行的sql
            flag:返回值类型，flag=lastrowid返回lastrowid，flag=rowcount返回rowcount
        """
        db = self.connect()
        cur = self._cursor(db)
        r = None
        try:
            cur.execute(sql)
            db.commit()
            r = 1
            if flag == 'lastrowid':
                r = cur.lastrowid
            elif flag == 'rowcount':
                r = cur.rowcount
        except pymysql.OperationalError:
            print("time: %s; Error on %s: %s" % (str(datetime.now()), self.host, sql[0: 200]))
            print(traceback.format_exc())
            r = -1
        except Exception:
            print("time: %s; Error to MySQL on %s: %s" % (str(datetime.now()), self.host, sql[0: 200]))
            print(traceback.format_exc())
            r = -1
        finally:
            cur.close()
            self.close(db)
        return r

    def save(self, table, data, mapping_fields=dict()):
        """
        将字典直接insert到数据库
        Args:
            table:字符串，插入目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            mapping_fields: 用于保持data字典的key与数据库字段的对应关系，
                            如果结果字典的某个key不包含在mapping_fields中，则将直接使用key作为字段名
        """

        if len(data) <= 0:
            return -1
        try:
            fields = ''
            values = ''
            for d in data:
                if d in mapping_fields:
                    fields += "`%s`," % (str(mapping_fields[d]))
                else:
                    fields += "`%s`," % (str(d))
                values += "'%s'," % (escape_string(str(data[d])))
            if len(fields) <= 0 or len(values) <= 0:
                return -1
            sql = "insert ignore into %s(%s) values(%s)" % (table, fields[:-1], values[:-1])
            return self.do(sql)
        except Exception:
            print(traceback.format_exc())
            return -1

    def saveMany(self, sql, values, flag='lastrowid'):
        """
        执行sql，insert/delete/update操作
        :param table: 'insert into table(id,name) values(%s,%s)'
        :param names: 字符串
        :param values:[{1,2},{1,2},...]  是一个列表，列表中的每一个元素必须是元组！！！
        :param flag:
        :return:
        """
        db = self.connect()
        cur = self._cursor(db)
        r = None
        try:
            cur.executemany(sql, values)
            db.commit()
            r = 1
            if flag == 'lastrowid':
                r = cur.lastrowid
            elif flag == 'rowcount':
                r = cur.rowcount

        except pymysql.OperationalError:
            print("Error connecting to MySQL on %s: %s" % (self.host, sql[0:255]))
            print(traceback.format_exc())
            r = -1
        except Exception:
            print("Error connecting to MySQL on %s: %s" % (self.host, sql[0:255]))
            print(traceback.format_exc())
            r = -1
        finally:
            cur.close()
            self.close(db)
        return r


    def update(self, table, data, field, mapping_fields=dict()):
        """
        将字典直接update到数据库
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据该字段判断是否为同一条记录，作为where条件
            mapping_fields: 用于保持data字典的key与数据库字段的对应关系，
                            如果结果字典的某个key不包含在mapping_fields中，则将直接使用key作为字段名
        """
        if len(data) <= 0:
            return -1
        else:
            try:
                values = ''
                field_value = None
                for d in data:
                    key = d
                    if d in mapping_fields:
                        key = mapping_fields[d]
                    if key == field:
                        field_value = data[d]
                    values += "%s='%s'," % (str(key), escape_string(str(data[d])))
                if len(values) <= 0 or field_value is None:
                    return -1
                sql = "update " + table + " set " + values[:-1] + " where " + field + "='" + escape_string(str(field_value)) + "'"
                return self.do(sql, flag='rowcount')
            except Exception:
                print(traceback.format_exc())
                return -1

    def saveorupdate(self, table, data, field, mapping_fields=dict()):
        """
        将字典更新到数据库，如果已存在则update，不存在则insert
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据词字段判断是否为同一条记录，作为where条件
            mapping_fields: 用于保持data字典的key与数据库字段的对应关系，
                            如果结果字典的某个key不包含在mapping_fields中，则将直接使用key作为字段名
        """
        if len(data) <= 0:
            return -1
        try:
            field_value = None
            if field in data:
                field_value = data[field]
            else:
                for key in mapping_fields:
                    if mapping_fields[key] == field and key in data:
                        field_value = data[key]
            if field_value is None:
                return -1
            querysql = "select count(1) from " + table + " where " + field + "='" + escape_string(str(field_value)) + "'"
            ed = self.query(querysql)
            if ed and ed[0][0] > 0:
                return self.update(table, data, field, mapping_fields)
            else:
                return self.save(table, data, mapping_fields)
        except Exception:
            print(traceback.format_exc())
            return -1

    def close_all(self):
        """
        关闭连接池全部连接
        请在确保连接不再需要时确认
        :return:
        """
        if self.close_able:
            self.pool.close()


def test():
    db = {
        'host': 'localhost',
        'user': 'root',
        'password': '',
        'db': 'test'
    }
    mq = StoreMysqlPool(**db)
    print(mq.count('urls'))


if __name__ == "__main__":
    test()



# -*- coding: utf8 -*-
import json
import os
import re
import sys
import random
import time
import base64
import traceback
from unit.get_baidu_cookie import baiduCookie

PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
sys.path.append(PROJECT_PATH)
sys.path.append(os.path.join(PROJECT_PATH, 'keyword_pulldown_monitoring'))


class BaiduSpiderPublicFun(object):
    """
    解析百度移动端，获取片名，real_url,rank
    """

    def __init__(self):
        super(BaiduSpiderPublicFun, self).__init__()
        self.baidu_cookie = baiduCookie()

    # 获取移动端ua
    def get_mobile_useragent(self):
        mobile_useragent_list = [
            "Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1",
            "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Mobile Safari/533.36",
            "Mozilla/5.0 (Linux; U; Android 5.0.2; zh-CN; Letv X501 Build/DBXCNOP5501304131S) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 UCBrowser/10.10.0.800 U3/0.8.0 Mobile Safari/534.30",
            "Mozilla/5.0 (Linux; U; Android 5.0.2; zh-cn; Letv X501 Build/DBXCNOP5501304131S) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/6.7 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; Android 5.1.1; vivo X6S A Build/LMY47V) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/35.0.1916.138 Mobile Safari/537.36 T7/6.3 baiduboxapp/7.3.1 (Baidu; P1 5.1.1)",
            "Mozilla/5.0 (Linux; U; Android 4.3; zh-cn; N5117 Build/JLS36C) AppleWebKit/534.24 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.24 T5/2.0 baiduboxapp/7.0 (Baidu; P1 4.3)",
            "Mozilla/5.0 (iPhone; CPU iPhone OS 9_2_1 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/13D15 UCBrowser/10.9.15.793 Mobile",
            "Mozilla/5.0 (iPhone 6p; CPU iPhone OS 9_2_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/6.0 MQQBrowser/6.7 Mobile/13D15 Safari/8536.25 MttCustomUA/2",
            "Mozilla/5.0 (iPhone; CPU iPhone OS 9_2_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13D15 Safari/601.1",
            "Mozilla/5.0 (Linux; U; Android 4.1.2; zh-cn; GT-S7572 Build/JZO54K) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/6.7 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; U; Android 5.1.1; zh-cn; SM-J3109 Build/LMY47X) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/6.6 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; U; Android 4.4.4; zh-cn; Coolpad 8297-T01 Build/KTU84P) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/6.6 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; U; Android 5.1.1; zh-CN; MX4 Pro Build/LMY48W) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 UCBrowser/10.10.0.800 U3/0.8.0 Mobile Safari/534.30",
            "Mozilla/5.0 (Linux; Android 5.1; m2 note Build/LMY47D) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/40.0.2214.114 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; U; Android 5.1; zh-CN; m2 note Build/LMY47D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 UCBrowser/10.9.10.788 U3/0.8.0 Mobile Safari/534.30",
            "Mozilla/5.0 (Linux; U; Android 5.1; zh-cn; m2 note Build/LMY47D) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/6.6 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; U; Android 4.4.4; zh-cn; CHM-CL00 Build/CHM-CL00) AppleWebKit/534.24 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.24 T5/2.0 baiduboxapp/7.1 (Baidu; P1 4.4.4)",
            "Mozilla/5.0 (Linux; Android 5.0.1; HUAWEI GRA-TL00 Build/HUAWEIGRA-TL00) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/37.0.0.0 Mobile Safari/537.36 MxBrowser/4.5.9.3000",
            "Mozilla/5.0 (Linux; Android 5.0.1; HUAWEI GRA-CL00 Build/HUAWEIGRA-CL00) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/35.0.1916.138 Mobile Safari/537.36 T7/6.3 baiduboxapp/7.3.1 (Baidu; P1 5.0.1)",
            "Mozilla/5.0 (Linux; Android 5.0.2; Redmi Note 2 Build/LRX22G) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/35.0.1916.138 Mobile Safari/537.36 T7/6.3 baiduboxapp/7.3.1 (Baidu; P1 5.0.2)",
            "Mozilla/5.0 (Linux; Android 4.4.4; Che1-CL10 Build/Che1-CL10) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/35.0.1916.138 Mobile Safari/537.36 T7/6.3 baiduboxapp/7.3.1 (Baidu; P1 4.4.4)",
            "Mozilla/5.0 (Linux; U; Android 4.4.2; zh-cn; HUAWEI P6-C00 Build/HuaweiP6-C00) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/6.7 Mobile Safari/537.36",
            "Mozilla/5.0 (Linux; Android 4.3; R7007 Build/JLS36C) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/35.0.1916.138 Mobile Safari/537.36 T7/6.3 baiduboxapp/7.3.1 (Baidu; P1 4.3)",
            "Mozilla/5.0 (Linux; Android 5.1.1; KIW-CL00 Build/HONORKIW-CL00) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/35.0.1916.138 Mobile Safari/537.36 T7/7.1 baidubrowser/7.1.12.0 (Baidu; P1 5.1.1)",
        ]
        return random.choice(mobile_useragent_list)

    # 获取pc端ua
    def get_pc_useragent(self):
        pc_useragent_list = [
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.3",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.110 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.110 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.186 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.79 Safari/537.36 Edge/14.14393",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1",
            "Mozilla/5.0 (Windows NT 5.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        ]
        return random.choice(pc_useragent_list)

    # 获取pc端请求头
    def get_pc_headers(self):
        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'Host': 'www.baidu.com',
            'Connection': 'keep-alive',
            'Upgrade-Insecure-Requests': '1',
            'Pragma': 'no-cache',
            'Cache-Control': 'no-cache',
            "Sec-Fetch-Dest": "document",
            'User-Agent': self.get_pc_useragent(),
            "Cookie": self.get_cook("1"),
            # "Cookie": 'BAIDUID=8BDD3E2AD9CD40385EBDE14EF60838B3:FG=1; BIDUPSID=8BDD3E2AD9CD403861768216F9752CD4; PSTM=1591698880; BD_UPN=133352; MSA_WH=375_667; H_WISE_SIDS=148078_147939_153647_150685_150076_147091_141744_150083_151862_148867_151312_153682_150745_147280_153629_153435_153289_153755_151016_151559_153567_146574_148523_151032_127969_153227_146548_152902_152982_146652_154013_146732_153058_154001_150764_131423_152022_152116_114553_147527_152931_107320_152716_140367_153502_144966_154061_153116_154118_139883_153060_153951_147546_153985_148869_150667_154293_151944_110085_154151; plus_cv=1::m:49a3f4a6; COOKIE_SESSION=79448_0_4_4_12_6_0_0_2_3_0_1_79441_0_2_0_1597988999_0_1597988997%7C9%23139428_11_1597280665%7C8; BDORZ=B490B5EBF6F3CD402E515D22BCDA1598; plus_lsv=f56cb9af77cd7927; Hm_lvt_12423ecbc0e2ca965d84259063d35238=1596432006,1597041078,1597286977,1597829277; BDRCVFR[gltLrB7qNCt]=mk3SLVN4HKm; delPer=0; BD_CK_SAM=1; rsv_i=2e46h8wdTj%2BhIW6U2Gun2sXfuu%2B4N79z1KB5tZZqscDcn0kzK%2BlHTOt61lFWIg8HGIBm2u7uVMHMxQv0pu92gWvYzJY2VWA; bd_af=1; SE_LAUNCH=5%3A26632993; FEED_SIDS=3000066_2; PSINO=5; kleck=5b193409d1a773576320fcef7a87b3c5; H_PS_PSSID=1465_32571_32328_32351_32045_32116_26350_32495_32482; H_PS_645EC=c1f4AXgdjLxePW6H13Q2cdn5tSiEbdYfbbvvwuO4cnCIBkSOBht8ohKoxWI; WWW_ST=1598001346785',
        }
        return headers

    # 获取移动端请求头
    def get_mb_headers(self):
        headers = {
            'Host': 'm.baidu.com',
            'User-Agent': random.choice(self.get_mobile_useragent()),
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
            'Referer': 'https://m.baidu.com/',
            'Connection': 'keep-alive',
            'Upgrade-Insecure-Requests': '1',
            'Pragma': 'no-cache',
            'Cache-Control': 'no-cache',
            'Cookie': self.get_cook("2"),
        }
        return headers

    # 获取公用库百度cookie
    def get_cook(self, device):
        """
        获取有效的cookie
        :return:
        """
        if device == '1':
            key_name = 'pc'
        else:
            key_name = 'mb'
        for i in range(2):
            if self.baidu_cookie.totalCookies(key_name=key_name) > 0:
                cookie = str(self.baidu_cookie.getLastCookies(key_name=key_name)[0], encoding="utf-8")
                return cookie
            else:
                print('no cookie')
                return 'BAIDUID={}:FG=1'.format(time.time())
        return ''

    # 拆解任务url
    def judge_url(self, ckurl):
        """
        判断url是否是一个json字符串，如果是返回一个由url组成的list
        :param ckurl:
        :return:
        """
        url_list = list()
        if ckurl.startswith("["):
            try:
                ckurl = ckurl.replace("\\'", "\"")
                url_list = json.loads(ckurl)
            except:
                print(traceback.format_exc())
                print("-------------")
                print(ckurl)
                url_list.append(ckurl)
        else:
            ckurl = ckurl.replace("'", "\"")
            url_list.append(ckurl)
        return url_list

    # 格式化pc端入库html
    def format_pc_html_save(self, html):
        html = html.replace("https://www.baidu.com/img/flexible/logo/pc/result.png",
                            "img/flexible/logo/pc/result.png") \
            .replace("//www.baidu.com/img/flexible/logo/pc/result.png", "img/flexible/logo/pc/result.png") \
            .replace('https://www.baidu.com/img/flexible/logo/pc/result@2.png', "img/flexible/logo/pc/result@2.png") \
            .replace('//www.baidu.com/img/flexible/logo/pc/result@2.png', "img/flexible/logo/pc/result@2.png")
        return str(base64.b64encode(html.encode(encoding="utf-8")), encoding="utf-8")

    # 格式化移动端入库html
    def format_mb_html_save(self, html):
        html = html.replace('position: fixed; display: flex; bottom: 0px; left: 0px; z-index: 300;', 'display:none;') \
            .replace('https://www.baidu.com/img/flexible/logo/logo_web.png', "img/flexible/logo/logo_web.png") \
            .replace('//www.baidu.com/img/flexible/logo/logo_web.png', "img/flexible/logo/logo_web.png")
        return str(base64.b64encode(html.encode(encoding="utf-8")), encoding="utf-8")

    # 判断pc端dom元素是否异常
    def check_pc_dom_is_error(self, html):
        if html.find("</html>") < 0 or html.find('id="wrap"') >= 0 or (
                "<title>百度App</title>" in html and "拦截通用" in html) or (
                "<title>百度安全验证</title>" in html and "百度安全验证" in html) or html.find('页面不存在_百度搜索') >= 0 or html.find(
            'id="container"') < 0 or html.find('id="content_left"') < 0 or html.find('<title>') < 0:
            return True
        else:
            return False

    # 判断移动端dom元素是否异常
    def check_mb_dom_is_error(self, html):
        if html.find("</html>") < 0 or html.find('id="results"') < 0 or (
                "<title>百度App</title>" in html and "拦截通用" in html) or (
                "<title>百度安全验证</title>" in html and "百度安全验证" in html) or html.find('页面不存在_百度搜索') >= 0 \
                or html.find('<title>') < 0:
            return True
        else:
            return False


if __name__ == '__main__':
    pass



# -*- coding: utf8 -*-
import sys
import oss2


class StoreOSS(object):
    """
    阿里云OSS操作
    Args:
        key_id：用户访问密钥AccessKeyId，用于签名
        key_secret：用户访问密钥AccessKeySecret，用于签名
        endpoint：访问域名
        bucket_name：存储空间（Bucket）名称
        connect_timeout：连接超时时间
    """
    def __init__(self, key_id, key_secret, endpoint, bucket_name, connect_timeout=None):
        self.bucket = oss2.Bucket(oss2.Auth(key_id, key_secret), endpoint, bucket_name, connect_timeout)

    def put(self, key, data):
        """
        上传数据对象
        Args:
            key：上传到OSS的文件名
            data：bytes，str或file-like object，待上传的内容
        """
        result = self.bucket.put_object(key, data)
        return result

    def get(self, key, filename=None):
        """
        下载数据对象
        Args:
            key：下载对象在OSS的文件名
            filename：要保存的文件名，如果未传递则返回类文件对象
        """
        if filename is None:
            return self.bucket.get_object(key)
        else:
            return self.bucket.get_object_to_file(key, filename)

    def delete(self, key):
        """
        删除数据对象
        Args:
            key：对象在OSS的文件名
        """
        return self.bucket.delete_object(key)

    def batch_delete(self, key_list):
        """
        批量删除数据对象
        Args:
            key_list：数据对象在OSS的文件名list
        """
        for i in range(0, len(key_list), 1000):
            self.bucket.batch_delete_objects(key_list[i:i+1000])


def test():
    oss = StoreOSS("BWUNG5LjYOHpKyr1", "JWyFaaDz2ydzz1bvFRggIdjRknjXhx", 'http://oss-cn-shanghai.aliyuncs.com', "cluster2016")
    oss.put('test.txt', 'just a test')
    r = oss.get('test.txt')
    print(r.read())
    oss.delete('test.txt')


if __name__ == "__main__":
    test()



# -*- coding: utf8 -*-
import os
import sys


class UtilUseragent(object):
    """
    获取User-Agent
    """
    def __init__(self):
        pass

    @staticmethod
    def get(type='PC'):
        file_name = 'ua_pc_list.txt'
        if type == 'MOBILE':
            file_name = 'ua_mobile_list.txt'
        elif type == 'SPIDER':
            file_name = 'ua_spider_list.txt'
        f = open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file_name), 'r')
        uas = [l.strip() for l in f.readlines()]
        f.close()
        return uas


def test():
    um = UtilUseragent()
    ualist = um.get(type='MOBILE')
    print(len(ualist))
    print(ualist)

if __name__ == "__main__":
    test()


# -*- coding: utf8 -*-
import hashlib
import sys


class UtilMD5(object):
    """
    MD5工具类
    """

    @staticmethod
    def md5(s):
        m = hashlib.md5()
        m.update(s.encode("utf-8"))
        return m.hexdigest()


def test():
    print(UtilMD5.md5('wangmspider'))

if __name__ == '__main__':
    test()


# -*- coding: utf-8 -*-

import traceback
import re
from lxml.html import fromstring

# from lxml import etree
from lxml import html as etree
import re
# import HTMLParser
import json
import sys


"""
from lxml import etree
html = etree.HTML(text)
"""

class BaiduExtractor(object):
    """
    百度搜索页面解析器
    示例：
    """

    def __init__(self):
        pass

    def extractor(self, text):
        """
         获取pc排名数据
         response_status 0请求失败 1 请求成功 2 页面不全，封ip
         百度关键词列表页数据
            返回 页面所有内容
            2 ： 页面异常 或 不全百度相关结果数
            0 ：没有数据
            -1 ： 异常
        brand_area   品牌专区  h2 有两个
        brand_website  官网  a 两个 内容 全匹配 官网
        """
        if not isinstance(text, str):
            text = text.decode(encoding="utf-8", errors='ignore') # bytes to str  用于匹配字符串

        if text.find('location.href.replace') >= 0:
            return 2
        elif text.find(u'页面不存在_百度搜索') > -1 or text.find(u'很抱歉，没有找到与') > -1:
            return dict()
        elif text.find(u'id="container"') < 0 or text.find(u'id="content_left"') < 0:
            return 0
        else:
            ext_result = dict()
            baidu_product_cnt = 0
            aladdin_cnt = 0
            ad_cnt = 0
            results_num = 0
            nature_cnt = 0
            include = ''
            try:
                advert = re.findall(r'class="m">广告<', text)
                ad_cnt = len(advert) if advert else 0
                print("广告数: {}".format(ad_cnt))

                # tree = fromstring(text.decode("utf-8", "ignore"))
                tree = fromstring(text)     # py3 自动转换
                containers = tree.cssselect('div.c-container')  # *行代码
                if containers:
                    rank_result_list = list()
                    for container in containers:
                        toprank = 0
                        title = ''
                        des = ''  # 描述
                        evaluate = ""  # 评价
                        realaddress = ""
                        domain = ""
                        show_date = ""
                        special_mark = 0
                        snapshoot_url = ""
                        keep_red = dict()

                        class_name = container.get("class")
                        if class_name.find("result-op") > -1:
                            special_mark = 3

                        toprank = int(container.get('id'))
                        # 标题
                        titles = container.cssselect("h3")
                        if titles:
                            # title = str(self.get_text(titles[0])).encode("utf-8").replace(u"举报图片", "").strip()
                            title = str(self.get_text(titles[0])).replace(u"举报图片", "").strip()
                            keep_red['title'] = re.findall(r'<em>.*?</em>', title)

                            if title.find("官网") > -1:
                                special_mark = 2
                            elif title.find("品牌") > -1:
                                special_mark = 1

                        # 内容
                        des_list = container.cssselect("div.c-abstract")
                        if len(des_list) > 0:
                            # des = self.get_text(des_list[0])
                            des = etree.tostring(des_list[0], encoding="utf-8", method="text").decode(encoding="utf-8")
                            # keep_red['desc'] = re.findall(r'<em>.*?</em>', etree.tostring(des_list[0], encoding="utf-8", method="text")) # py2

                            keep_red['desc'] = re.findall(r'<em>.*?</em>', des)
                            show_date_eles = des_list[0].xpath('descendant::span')
                            if show_date_eles:
                                show_date = etree.tostring(show_date_eles[0], encoding="utf-8", method="text").strip().decode(encoding="utf-8")
                                show_date = str(show_date).split("-")[0].strip().replace(" ", "")

                        else:
                            des_list = container.cssselect("div.c-span18c-span-last")
                            if len(des_list) > 0:
                                des = self.get_text(des_list[0])
                                keep_red['desc'] = re.findall(r'<em>.*?</em>', des)

                                show_date_eles = des_list[0].xpath('descendant::span[@class="m"]')
                                if show_date_eles:
                                    show_date = etree.tostring(show_date_eles[0], encoding="utf-8",
                                                               method="text").strip().decode(encoding="utf-8")
                                    show_date = str(show_date).split("-")[0].strip().replace(" ", "")
                            else:
                                des_list = container.cssselect("div.c-gap-top-small")
                                if des_list:
                                    des = self.get_text(des_list[0])
                                    # keep_red['desc'] = re.findall(r'<em>.*?</em>', etree.tostring(des_list[0]))

                                    keep_red['desc'] = re.findall(r'<em>.*?</em>',
                                                                  etree.tostring(des_list[0], encoding="utf-8",
                                                                    method="text").decode(encoding="utf-8"))
                                    show_date_eles = des_list[0].xpath('descendant::span')
                                    if show_date_eles:
                                        show_date = etree.tostring(show_date_eles[0], encoding="utf-8",
                                                                   method="text").strip().decode(encoding="utf-8")
                                        show_date = str(show_date).split("-")[0].strip().replace(" ", "")

                        realaddress_list = container.cssselect("a")
                        if len(realaddress_list) > 0:
                            realaddress = realaddress_list[0].get("href")
                        # domain 显示url
                        domain_list = container.cssselect("a.c-showurl")
                        if len(domain_list) > 0:
                            # domain = self.get_text(domain_list[0]).strip()
                            # keep_red['url'] = re.findall(r'<b>.*?</b>', etree.tostring(domain_list[0]))   # py2

                            domain = etree.tostring(domain_list[0], encoding="utf-8", method="text").decode(
                                encoding="utf-8")
                            if domain.find("}") > -1:
                                domain = domain.split("}")[1]
                            keep_red['url'] = re.findall(r'<b>.*?</b>', domain)
                        else:
                            domain_list = container.cssselect("span.c-showurl")
                            if len(domain_list) > 0:
                                domain = self.get_text(domain_list[0]).strip()
                                if domain.find("}") > -1:
                                    domain = domain.split("}")[1]
                                keep_red['url'] = re.findall(r'<b>.*?</b>', domain)

                        if domain == "":
                            domain_list = container.cssselect("div.g")
                            if len(domain_list) > 0:
                                domain = self.get_text(domain_list[0]).strip()
                                if domain.find("}") > -1:
                                    domain = domain.split("}")[1]
                                keep_red['url'] = re.findall(r'<b>.*?</b>', domain)
                        if domain == "":
                            domain_list = container.cssselect("span.g")
                            if len(domain_list) > 0:
                                domain = self.get_text(domain_list[0]).strip()
                                if domain.find("}") > -1:
                                    domain = domain.split("}")[1]
                                keep_red['url'] = re.findall(r'<b>.*?</b>', domain)

                        if title.find(u"的最新相关信息") > -1:
                            domain = "baidu"

                        if domain.find(".baidu.") > -1:
                            # baidu_product_cnt += 1
                            special_mark = 4
                        else:
                            nature_cnt += 1

                        if domain.find("image.baidu.") > -1:
                            des = "百度图片"
                        if not des:
                            try:
                                if domain_list:
                                    domain_list[0].getparent().remove(domain_list[0])
                                if titles:
                                    titles[0].getparent().remove(titles[0])
                                des = str(self.get_text(container)).split("查看更多")[0]
                                if domain.find("tieba.baidu.") > -1:
                                    # 百度贴吧
                                    des = des.split("杜绝广告")[1]
                            except:
                                des = ""
                        item = dict()

                        div_html = etree.tostring(container, encoding='utf-8', method='html').decode(encoding="utf-8")
                        if div_html.find('aladdin') > 0 or div_html.find('alading') > 0:
                            item['alading'] = 1
                        else:
                            item['alading'] = 0
                        item['show_url'] = domain
                        item['rank'] = toprank
                        item['show_title'] = title
                        item['show_desc'] = des
                        item['url'] = realaddress
                        item["special_mark"] = special_mark
                        item["show_date"] = show_date
                        # item["snapshoot_url"] = snapshoot_url
                        # if keep_red:
                        #     for k, v in keep_red.items():
                        #         keep_red[k] = [self.html_parser.unescape(i).replace("<em>", "").replace("</em>", "")
                        #                            .replace("<b>", "").replace("</b>", "")
                        #                        for i in list(set(v))]
                        # item["keep_red"] = json.dumps(keep_red)

                        rank_result_list.append(item)
                else:
                    return 0
                # ext_result["nature_cnt"] = len(containers)
                # ext_result["relate_cnt"] = results_num  # 相关搜索数
                # ext_result["ad_cnt"] = ad_cnt
                # ext_result["aladdin_cnt"] = aladdin_cnt
                # ext_result["baidu_product_cnt"] = baidu_product_cnt  # 百度自由产品数
                # ext_result["include"] = include  # 收录数
                # ext_result["data"] = rank_result_list

                # include_else = re.findall(r'找到相关结果约(.*?)个', text)
                # if include_else:
                #     include = include_else[0]

                results = re.findall(r'百度为您找到相关结果约(.*?)个', text)
                if results:
                    include = int(str(results[0]).replace(",", ""))
                    print("百度相关结果数: {}".format(include))

                related_else = tree.xpath('.//div[@id="rs"]//th/a/text()')
                keyword_list = list()
                for keyword in related_else:
                    keyword_list.append({"keyword": str(keyword)})
                    # keyword_list = ','.join(keyword_list)
                return rank_result_list, include, keyword_list
            except Exception:
                print(traceback.format_exc())
                return -1

    def get_text(self, elem):
        rc = []
        for node in elem.itertext():
            rc.append(node.strip())
        return ''.join(rc)

    def remove_characters(self, previou_url):
        if previou_url.startswith("https://"):
            previou_url = previou_url.replace("https://", "")
        if previou_url.startswith("http://"):
            previou_url = previou_url.replace("http://", "")
        if previou_url.endswith("/"):
            previou_url = previou_url[0:len(previou_url) - 1]
        return previou_url

    def remove_special_characters(self, domain):
        domain = domain.replace("<b>", "")
        domain = domain.replace("</b>", "")
        domain = domain.replace("&nbsp", "")
        domain = domain.replace("....", "")
        domain = domain.replace("...", "")
        return domain


if __name__ == '__main__':
    extractor = BaiduExtractor()
    with open("html_py2.txt", 'r', encoding="utf-8") as     f:
        text = f.read()

    # str -> bytes
    text = text.encode(encoding='utf-8')
    extractor_result = extractor.extractor(text)
    # print(extractor_result)
    result, include, keyword_list = extractor_result
    print(result)
    for r in result:
        for k in r:
            print(k, r[k])
        print("-"*20)

    print(include)
    print(keyword_list)



# -*- coding: utf8 -*-
import sys


class UtilUrl(object):
    """
    url处理工具类
    """

    @staticmethod
    def get_url(from_url, cur_url):
        """
        拼接完整的url
        Args:
            from_url:源url
            cur_url:要处理的url，可能是完整的url，也可能是以/、?等开始的不完整的url
        """
        if cur_url.startswith('http'):
            return cur_url
        elif cur_url.startswith('/'):
            index = from_url.find('/', 8)
            return from_url[0:index] + cur_url
        elif cur_url.startswith('?'):
            index = from_url.find('?')
            return from_url[0:index] + cur_url
        else:
            index = from_url.rfind('/')
            return from_url[0:index + 1] + cur_url


# -*- coding: utf8 -*-

from elasticsearch import Elasticsearch
import elasticsearch.helpers
from datetime import time


class StoreEs(object):
    def __init__(self, host="localhost", port=9200, params=None):
        if params is not None:
            self.es = Elasticsearch("{0}:{1}".format(host, port),
                                    **params)
        else:
            self.es = Elasticsearch("{0}:{1}".format(host, port))

    def save(self, index, doc_type, body, id=None, params=None):
        field = dict()
        field['index'] = index
        field['doc_type'] = doc_type
        field['body'] = body
        if id:
            field['id'] = id
        if params:
            field['params'] = params
        self.es.index(**field)

    def batch_save(self, index, doc_type, body):
        for i, val in enumerate(body):
            while True:
                try:
                    self.save(index, doc_type, val)
                    break
                    pass
                except:
                    time.sleep(1)
                    continue

    def bulk(self, index, doc_type, body):
        actions = [
            {
                '_op_type': 'index',
                '_index': index,
                '_type': doc_type,
                '_source': d
            }
            for d in body
        ]
        elasticsearch.helpers.bulk(client=self.es, actions=actions)

    def search(self, index, doc_type, body=None, params=None):
        if params is None:
            res = self.es.search(index=index, doc_type=doc_type, body=body)
        else:
            res = self.es.search(index=index, doc_type=doc_type, body=body, params=params)
        return res

    def get(self, index, doc_type, id):
        return self.es.get(index=index, doc_type=doc_type, id=id)

    def update_by_query(self, index, doc_type=None, body=None, params=None):
        return self.es.update_by_query(index=index, doc_type=doc_type, body=body, params=params)

    def delete(self, index, doc_type, id):
        self.es.delete(index=index, doc_type=doc_type, id=id)

# def save(html, es, q):
#     for i in range(5000):
#         id = str(uuid.uuid1())
#         val = {
#             'result': html,
#             'iden': i,
#         }
#         es.save('yuqing', 'urls_11', val, id=id)
#         q.put(id)

# def delete(es, q):
#     import time
#     from Queue import Empty
#     import traceback
#     while True:
#         try:
#             id = q.get_nowait()
#             # r = es.get('yuqing', 'urls_11', id)
#             # print r['_source']['iden']
#             r = es.es.delete('yuqing', 'urls_11', id)
#             print r
#         except Empty:
#             time.sleep(2)
#             print 'sleep'
#         except Exception, e1:
#             print(traceback.format_exc())


if __name__ == '__main__':
    # import uuid
    # from Queue import Queue
    # from threading import Thread
    from download_center.store.store_es import StoreEs

    config = {
        'host': '115.159.158.157',
        'port': 9200,
        'params': {
            'http_auth': ('admin', 'Iknowthat221@')
        }
    }
    try:
        es = StoreEs(**config)
        result = es.get("content_center", "tiezi", "AWBaEjLkLUY1KU5jKSwu")
    except:
        pass





# -*- coding: utf8 -*-
import json
from datetime import datetime
from datetime import timedelta
import traceback
import base64
import time
import hashlib
from threading import Timer

import pymysql

import urllib3
urllib3.disable_warnings()
import requests

from pymysql.converters import escape_string

reqs = requests.session()
reqs.keep_alive = False

from download_center.new_spider.downloader import config  # py3
import sys


def util_md5(s):
    m = hashlib.md5()
    m.update(s)
    return m.hexdigest()


class Downloader(object):
    """
    通用下载器
    """

    def __init__(self, set_mode='db', get_mode='db', store_type=5):     # 兼容
        self.t = Timer(60*10, self.reset_ip)
        pass

    def reset_ip(self):
        """
        当接口调用失败后，重新ping以下网络情况，并更新接口地址
        :return:
        """
        try:
            ip_type = 1 if config.ping(config.DOWNLOADER_CENTER_INNER_IP) else 2
            if ip_type == 2 :
                self.t.start()
            else:
                self.t.cancel()
            response = reqs.post(config.TASK_SCHEDULER_IP.format(config.IP_HOST), data={"type": ip_type}, timeout=config.REQUEST_TIMEOUT)
            config.downloader_ip = str(response.content).strip()
            print("current downloader_ip: {}".config.downloader_ip)
        except:
            time.sleep(60)
            print(traceback.format_exc())

    def downloader_add_black_ip(self, ip, type, expire_time=6):
        try:
            data = {"ip": ip, "type": type, "expire_time": expire_time}
            reqs.post(config.ADD_BLACK_IP.format(config.downloader_ip), data=data, timeout=config.REQUEST_TIMEOUT)
            return True
        except:
            print(traceback.format_exc())
            return False

    def validate_accout(self, user="", password=""):
        try:
            ip_type = 1 if config.ENVIR == 'inner' else 2
            data = {"user": user, "password": password, 'type': ip_type}
            for i in range(2):
                try:
                    r = reqs.post(config.VALIDATE_ACCOUNT_URL, data=data, timeout=config.REQUEST_TIMEOUT)
                    rdata = json.loads(r.text)
                    break
                except:
                    time.sleep(config.REQUEST_RETYR_SLEEP)
                    # print(traceback.format_exc())
                    rdata = {'status': 0,'msg':'login failure'}

            if rdata["status"] == 0:
                print(rdata["msg"])
                return False
            else:
                config.downloader_ip = rdata["ip"]
                return rdata["user_id"]
        except:
            print("datetime: {}; error: {}".format(str(datetime.now()), str(traceback.format_exc())))
            return False

    def set(self, request):
        """
        放任务
        Returns:
            正常: 返回字典，key值为每个url，value值0（失败）、  1（成功） 其它失败
            出错: 0 参数问题  -2 地域问题  -1 错误 1 正常

        請求異常 超時  返回正常， 其它根據返回狀態
        """
        try:
            data = {"user_id": request.user_id,
                    "headers": json.dumps(request.headers),
                    "config": json.dumps(request.config),
                    "urls": json.dumps(request.urls)
                    }
            for i in range(3):
                try:
                    rdata = reqs.post(config.DOWNLOADER_SENDTASK.format(config.downloader_ip), data=data, timeout=config.REQUEST_TIMEOUT)
                    r = json.loads(rdata.text)
                    break
                except:
                    # print(traceback.format_exc())
                    time.sleep(config.REQUEST_RETYR_SLEEP)
                    r = {'status': 0, 'msg': 'set failure'}

            if r["status"] == 1:
                rdata = json.loads(r["rdata"])
                for i, k in enumerate(rdata):
                    request.urls[i]["unique_md5"] = k
            else:
                print("set msg: {}".format(r["msg"]))
            return int(r["status"])
        except Exception:                                         # data exception
            print("datetime: {}; error: {}".format(str(datetime.now()), str(traceback.format_exc())))
            time.sleep(config.REQUEST_RETYR_MAX_SLEEP)

            self.reset_ip()
            for url in request.urls:
                if 'unique_key' in url.keys():
                    md5 = util_md5(escape_string(url['url']) + str(url['unique_key']))
                else:
                    md5 = util_md5(escape_string(url['url']))
                url['unique_md5'] = md5
            return -1

    def get(self, request):
        """
        向下载中心请求特定url的结果
        模式，redis->直接查询redis数据库，db->直接查询数据库，http->通过http接口查询
        Args:
            request: SpiderRequest对象
        """
        # noinspection PyBroadException
        try:
            data = {"user_id": request.user_id,
                    "config": json.dumps(request.config),
                    "urls": json.dumps(request.urls)
                    }
            for i in range(3):
                # noinspection PyBroadException
                try:
                    rdata = reqs.post(config.DOWNLOADER_GETRESULT.format(config.downloader_ip), data=data, timeout=config.REQUEST_TIMEOUT)
                    r = json.loads(rdata.text)
                    break
                except Exception:
                    r = {'status': 0,'msg':'reqs failure'}
                    # print(traceback.format_exc())
                    time.sleep(config.REQUEST_RETYR_SLEEP)

            if r["status"] == 1:
                unique_md5_results = json.loads(r["rdata"])
                for unique_md5 in unique_md5_results:
                    if 'result' in unique_md5_results[unique_md5] and unique_md5_results[unique_md5]['result']:
                        html = base64.b64decode(unique_md5_results[unique_md5]['result'])
                        if html and isinstance(html, bytes):
                            try:
                                html = html.decode(encoding="utf-8", errors='ignore')  # bytes to str
                            except:
                                pass
                        unique_md5_results[unique_md5]['result'] = html
                return unique_md5_results
            else:
                print("get error msg: {}".format(r["msg"]))
                return r["status"]
        except Exception:
            print("datetime: {}; error: {}".format(str(datetime.now()), str(traceback.format_exc())))
            time.sleep(config.REQUEST_RETYR_MAX_SLEEP)

            self.reset_ip()
            rdata = dict()
            for url in request.urls:
                rdata[url['unique_md5']] = {"status": 0}
            return rdata

class SpiderRequest(object):

    __slots__ = ['user_id', 'headers', 'config', 'urls']    # save memory

    def __init__(self, user_id=None, headers=dict(), config=dict(), urls=list()):
        self.user_id = user_id
        self.headers = headers
        self.config = config
        self.urls = urls

    def set_headers_key(self, key, value):
        self.headers[key] = value

    def set_headers(self, header):
        self.headers = header

    def set_config_key(self, key, value):
        self.headers[key] = value

    def set_config(self, set_configs):
        self.headers = set_configs


def main():
    spider = Downloader()
    # spider.validate_accout(user="test", password="Welcome#1")
    # spider.validate_accout(user="sunxiang", password="sxspider")
    spider.reset_ip()
    # req = SpiderRequest()
    # print(spider.get(req))

    # spider.downloader_add_black_ip("fihf", '11', expire_time=6)
    print(config.downloader_ip)


if __name__ == '__main__':
    main()





# -*- coding: utf8 -*-

from download_center.new_spider.downloader.downloader import Downloader
from download_center.new_spider.downloader.downloader import SpiderRequest
from download_center.new_spider.downloader.html_local_downloader import HtmlLocalDownloader
from download_center.new_spider.util.util_useragent import UtilUseragent
# from Queue import LifoQueue
from queue import Queue
from queue import Empty
from threading import Thread
import time
import traceback
import objgraph
from datetime import datetime
import uuid
from prettytable import PrettyTable
import sys


class BaseSpider(object):
    """
    基本的爬虫类
    该爬虫维护四个队列：待发送队列sending_queue、已发送队列sended_queue
    取回结果队列response_queue、待存储队列store_queue
    发送的对象统一封装成SpiderRequest对象
    启动时，通过start_requests方法构造一批待发送的SpiderRequest对象送到待发送队列里面
    发送线程send_requests依次从sending_queue队列取值并发送，获取的结果通过deal_request_results来处理
    如果发送成功，将SpiderRequest对象送入已发送队列sended_queue，如果发送失败则重新回到待发送队列sending_queue
    取结果线程get_response依次从sended_queue队列取值并发送，获取的结果放至取回结果队列response_queue，
    deal_response_results从取回结果队列response_queue取出来处理，处理结束后可把结果放至待存储队列store_queue异步存储，也可在
    deal_response_results直接调用stores的存储方法直接存储
    如果抓取成功，将对应结果存入数据库，如果发送失败则重新回到已发送队列sended_queue
    """

    def __init__(self, remote=True):
        self.remote = remote
        self.pc_user_agents = UtilUseragent.get()
        self.mb_user_agents = UtilUseragent.get(type='MOBILE')
        self.sending_queue = Queue()
        self.sended_queue = Queue()
        self.response_queue = Queue()
        self.store_queue = Queue()
        self.user_id = 0
        self.stores = self.get_stores()
        self.downloader = self.get_downloader()
        self.sended_queue_max = 5000    # sended_queue 最大值
        self.response_queue_max = 1500  # response_queue 最大值
        self.store_queue_max = 2000     # store_queue 最大值
        self.url_repeat = True          # 默认重复的url重发 false 不重发
        self.downloader_ip = None
        self.thread_wait = 0            # 线程等待时间

    def get_user_password(self):
        """
        设置用户名和密码，由子类实现。用于爬取前验证是否合法
        """
        raise NotImplementedError()

    def validate_user(self):
        """
        验证用户
        """
        if self.remote:
            user, password = self.get_user_password()
            rdata = self.downloader.validate_accout(user=user, password=password)
            if rdata:
                self.user_id = rdata
                print("user: {}; password: {} user validate success".format(user, password))
            else:
                raise RuntimeError('用户验证失败')
                sys.exit()
        else:
            pass

    def get_downloader(self):
        """
        设置下载器类型，默认为Downloader
        Return:
            SpiderDownloader
        """
        if self.remote:
            return Downloader()
        else:
            print("local downloader")
            return HtmlLocalDownloader()

    def start_requests(self):
        """
        初始化待发送请求队列，由子类实现。拼装一串SpiderRequest对象并送到sending_queue队列中
        """
        raise NotImplementedError()

    def is_finish(self):
        """
        根据相关队列是否全都为空来判断任务处理结束
        """
        return self.sending_queue.empty()and self.sended_queue.empty() \
            and self.response_queue.empty() and self.store_queue.empty()

    def send_requests(self, max_idle_time):
        """
        发送请求。将sending_queue队列中的SpiderRequest对象通过downloader发送到下载中心
        """
        start_time = time.time()
        while True:
            try:
                if self.sended_queue.qsize() < self.sended_queue_max and self.response_queue.qsize() < self.response_queue_max:
                    request = self.sending_queue.get(timeout=1)
                    if request.user_id is None:
                        request.user_id = self.user_id
                    results = self.downloader.set(request)
                    self.deal_request_results(request, results)
                    start_time = time.time()
                    self.send_wait()
                else:
                    time.sleep(10)
            except Empty:
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(10)
            except Exception:
                print(traceback.format_exc())

    def send_wait(self):
        """
        发送等待, 控制发往下载中心的速率
        """
        if self.thread_wait != 0:
            time.sleep(self.thread_wait)
        else:
            if self.sended_queue.qsize() > 8000:
                time.sleep(10)
            elif self.sending_queue.qsize() < 1000:
                time.sleep(1)
            else:
                time.sleep(0.1)

    def get_wait(self):
        """
        获取结果等待, 控制发往处理队列的速率
        """
        if self.thread_wait != 0:
            time.sleep(self.thread_wait)
        else:
            if self.response_queue.qsize() > 4000:
                time.sleep(10)
            elif self.sended_queue.qsize() < 1000:
                time.sleep(1)
            else:
                time.sleep(0.1)

    def get_response(self, max_idle_time):
        """
        获取url爬取结果。将sended_queue队列中的SpiderRequest对象通过downloader到下载中心去获取抓取到的html
        """
        start_time = time.time()
        while True:
            try:
                if self.response_queue.qsize() < self.response_queue_max:
                    request = self.sended_queue.get(timeout=1)
                    results = self.downloader.get(request)
                    #status = results[request.urls[0]['unique_md5']]
                    self.response_queue.put((request, results))
                    start_time = time.time()
                    self.get_wait()     # wait confirm
                else:
                    time.sleep(10)
            except Empty:
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(10)
            except Exception:
                print(traceback.format_exc())

    def deal_response(self, max_idle_time):
        """
        从结果队列response_queue中取出结果进行处理
        """
        start_time = time.time()
        while True:
            try:
                if self.store_queue.qsize() < self.store_queue_max:
                    request, results = self.response_queue.get(timeout=1)
                    try:
                        self.deal_response_results(request, results, self.stores)
                    except Exception:
                        print(traceback.format_exc())
                    start_time = time.time()
                else:
                    time.sleep(10)
            except Empty:
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(10)
            except Exception:
                print(traceback.format_exc())

    def store_results(self, max_idle_time):
        """
        从store_queue里面取出待存储的数据进行存储
        """
        start_time = time.time()
        while True:
            try:
                results = self.store_queue.get(timeout=1)
                self.to_store_results(results, self.stores)
                start_time = time.time()
            except Empty:
                if max_idle_time == -1:
                    pass
                elif start_time + max_idle_time < time.time():
                    if self.is_finish():
                        break
                time.sleep(10)
            except Exception:
                print(traceback.format_exc())

    def to_store_results(self, results, stores):
        """
        存储结果，由子类实现。
        Args:
            results:处理response后的结果
            stores:list，可能会用到的存储器（SpiderStore）列表
        """
        pass

    def get_stores(self):
        """
        设置存储器，供deal_response_results调用
        Return:
            list,成员为SpiderStore对象
        """
        stores = list()
        return stores

    def deal_request_results(self, request, results):
        if results == 0:
            print('params exception request failure urls: {}'.format(request.urls))
        elif results == -2:
            print('no district urls: {}'.format(request.urls))
        elif results == -1:
            print("set request failure  urls: {}".format(request.urls))
            self.sended_queue.put(request)
        else:
            self.sended_queue.put(request)

    def deal_response_results(self, request, results, stores):
        if results == 0:
            print("deal_response_results results = 0 ")
            return False
        else:
            urls = list()       # 重新去获取的url
            failed_urls = list()    # 失败重发的url
            for u in request.urls:
                url = u['unique_md5']
                if url in results:
                    result = results[url]
                    task_status = str(result['status'])
                    if task_status in ['2', '3']:
                        ret_failed_urls = self.deal_response_results_status(task_status, u, result, request)
                        if ret_failed_urls:
                            failed_urls = failed_urls + ret_failed_urls
                    else:
                        urls.append(u)
                else:
                    print('url send failure unique_md5:{}; url have:{}'.format(url, u["url"]))
                    if self.url_repeat:
                        failed_urls.append(u)
            if len(urls) > 0:
                request.urls = urls
                self.sended_queue.put(request)
            if len(failed_urls) > 0:
                new_request = SpiderRequest(headers=request.headers, config=request.config)
                new_request.urls = failed_urls
                self.sending_queue.put(new_request)
                new_request = None

    def deal_response_results_status(self, task_status, url, result, request):
        """
            处理 task_status 是2,3的任务  重试返回数组， 若重试需切换headers内容需自行定义
        :param task_status:
        :param url:
        :param result:
        :param request:
        :return:
        """
        raise NotImplementedError()

    def record_log(self):
        """
        记录抓取日志，用于调整各个线程参数设置
        """
        while True:
            table = PrettyTable(["x", "y"])
            table.add_row(["datetime", str(datetime.now())])
            table.add_row(["sending_queue", self.sending_queue.qsize()])
            table.add_row(["sended_queue", self.sended_queue.qsize()])
            table.add_row(["response_queue", self.response_queue.qsize()])
            table.add_row(["store_queue", self.store_queue.qsize()])
            table.reversesort = True
            print(table)
            del table
            # objgraph.show_most_common_types()
            time.sleep(30)
            if self.is_finish():
                break

    @staticmethod
    def get_unique_key():
        """
        生成唯一标识
        :return:
        """
        return str(uuid.uuid1())

    def add_black_ip(self, ip, type, expire_time=6):
        """
        记录IP黑名单
        使用黑名单功能需在发送请求时在config设置param参数中配置任务类型
        config : {param: {'task_type': 1}}
        Args:
            ip: IP地址
            type: 指定任务类型有效
            expire_time: 超时时间(小时)
        """
        self.downloader.downloader_add_black_ip(ip, type, expire_time=expire_time)

    def clear_task_pool(self):
        """
        清空当前用户已发往下载中心的任务
        :return:
        """
        pass

    def after_deal_response(self, request, results):
        """
        抓取结果的处理逻辑后的后续操作，按需使用，默认不作任何操作
        :param request:
        :param results:
        :return:
        """
        pass

    @staticmethod
    def retry(u, count):
        retry_urls = list()
        if u.get("re_send", 0) < int(count):
            u["re_send"] = u.get("re_send", 0) + 1
            retry_urls.append(u)
        else:
            print("conf_search_count > {};url: {}".format(count, u["url"]))
        return retry_urls

    def run(self, send_num=0, get_num=0, deal_num=0, store_num=0,
            send_idle_time=600, get_idle_time=600,
            deal_idle_time=600, store_idle_time=600, record_log=False, spider_count=0):
        """
        爬虫启动入口
        Args:
            send_num:发送请求线程数，默认为0
            get_num:获取结果线程数，默认为0
            deal_num:处理结果线程数，默认为0
            store_num:存储结果线程数，默认为0
            send_idle_time:发送请求线程超过该时间没有要发送的请求就停止，-1永不停止
            get_idle_time:获取结果线程超过该时间没有要获取的结果就停止，-1永不停止
            deal_idle_time:处理结果线程超过该时间没有要处理的结果就停止，-1永不停止
            store_idle_time:存储结果线程超过该时间没有要存储的结果就停止，-1永不停止
            record_log:定时记录各个队列大小，便于分析抓取效率
            spider_count: 设置每小时发送量
        """
        self.validate_user()
        thread_start = Thread(target=self.start_requests)
        thread_start.start()

        if send_num == 0:        # 用户自行配置
            if spider_count == 0:
                print("not set send_num and spider_count")
                sys.exit()
            else:
                send_num = 5
                get_num = 7
                if deal_num == 0:
                    deal_num = 5
                if store_num == 0:
                    store_num = 5
                self.thread_wait = round(3600/(float(spider_count)/send_num), 2)

        print("send_num: {}; get_num: {}; deal_num: {}; store_num: {}; spider_count: {}".format(send_num, get_num, deal_num, store_num, spider_count))
        threads = list()
        for i in range(0, send_num):
            threads.append(Thread(target=self.send_requests, args=(send_idle_time,)))
        for i in range(0, get_num):
            threads.append(Thread(target=self.get_response, args=(get_idle_time,)))
        for i in range(0, deal_num):
            threads.append(Thread(target=self.deal_response, args=(deal_idle_time,)))
        for i in range(0, store_num):
            threads.append(Thread(target=self.store_results, args=(store_idle_time,)))

        if record_log:
            thread = Thread(target=self.record_log)
            thread.setDaemon(True)
            threads.append(thread)
        for thread in threads:
            thread.start()



# -*- coding:utf-8 -*-
"""
@Time : 2021/10/15 13:09
@Author: LiADai(李晗)
@File : huawei_bankuai_page_spider.py
@ 版块下帖子链接/帖子title抓取，并标注发帖时间/是否加V或精华、热门帖
"""
import ssl

import requests

from demo.spider import config

ssl._create_default_https_context = ssl._create_unverified_context
import logging
import os
import urllib3
import warnings
import sys
from queue import Queue
import random
import time
from download_center.new_spider.downloader.downloader import SpiderRequest
from download_center.new_spider.spider.basespider import BaseSpider
from demo.store.py_store_mysql_pool import StoreMysqlPool

urllib3.disable_warnings()
warnings.filterwarnings('ignore')
file_name = os.path.basename(__file__).split('.')[0]
PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(PROJECT_PATH)
logging.basicConfig(filename='{}/logs/{}.log'.format(PROJECT_PATH, file_name), level=logging.INFO,
                    format='%(asctime)s - %(levelname)s - %(name)s : %(message)s')

class mum_creat_tool(BaseSpider):
    def __init__(self, remote=True):
        super(mum_creat_tool, self).__init__(remote)
        #版块下的数据，只允许访问2K条，登录也没用
        self.dataList = [
            {"name":"荣耀Magic系列","path":"https://club.hihonor.com/cn/forum-3965-{}.html?filter=lastpost","page":369},
            {"name":"荣耀数字系列","path":"https://club.hihonor.com/cn/forum-3484-{}.html?filter=lastpost","page":183},
            {"name": "荣耀V系列", "path": "https://club.hihonor.com/cn/forum-3159-{}.html?filter=lastpost", "page": 69},
            {"name": "荣耀X系列", "path": "https://club.hihonor.com/cn/forum-4280-{}.html?filter=lastpost", "page": 45},
            {"name": "荣耀Play系列", "path": "https://club.hihonor.com/cn/forum-3666-{}.html?filter=lastpost", "page": 14},
            {"name": "荣耀畅玩系列", "path": "https://club.hihonor.com/cn/forum-4526-{}.html?filter=lastpost", "page": 4},
            {"name": "问题反馈", "path": "https://club.hihonor.com/cn/forum-4508-{}.html?filter=lastpost", "page": 69},
            {"name": "荣耀亲选", "path": "https://club.hihonor.com/cn/forum-154-{}.html?filter=lastpost", "page": 2},
            {"name": "智能家居", "path": "https://club.hihonor.com/cn/forum-1835-{}.html?filter=lastpost", "page": 4},
            {"name": "智能穿戴", "path": "https://club.hihonor.com/cn/forum-4301-{}.html?filter=lastpost", "page": 10},
            {"name": "公测内测", "path": "https://club.hihonor.com/cn/forum-4149-{}.html?filter=lastpost", "page": 3},
            {"name": "Magic UI 4.0", "path": "https://club.hihonor.com/cn/forum-455-{}.html?filter=lastpost","page": 7},
            {"name": "使用技巧", "path": "https://club.hihonor.com/cn/forum-3667-{}.html?filter=lastpost", "page": 4},
            {"name": "荣耀云服务", "path": "https://club.hihonor.com/cn/forum-4522-{}.html?filter=lastpost", "page": 1},
            {"name": "爱美食", "path": "https://club.hihonor.com/cn/forum-713-{}.html?filter=lastpost", "page": 46},
            {"name": "爱旅行", "path": "https://club.hihonor.com/cn/forum-721-{}.html?filter=lastpost", "page": 41},
            {"name": "爱运动", "path": "https://club.hihonor.com/cn/forum-910-{}.html?filter=lastpost", "page": 47},
            {"name": "爱主题", "path": "https://club.hihonor.com/cn/forum-1053-{}.html?filter=lastpost", "page": 113},
            {"name": "爱游戏", "path": "https://club.hihonor.com/cn/forum-4400-{}.html?filter=lastpost", "page": 31},
            {"name": "爱摄影", "path": "https://club.hihonor.com/cn/forum-141-{}.html?filter=lastpost", "page": 470},
            {"name": "慢生活", "path": "https://club.hihonor.com/cn/forum-64-{}.html?filter=lastpost", "page": 12},
            {"name": "荣耀电竞堂", "path": "https://club.hihonor.com/cn/forum-1046-{}.html?filter=lastpost", "page": 1},
            {"name": "王者荣耀", "path": "https://club.hihonor.com/cn/forum-4513-{}.html?filter=lastpost", "page": 4},
            {"name": "使命召唤手游", "path": "https://club.hihonor.com/cn/forum-4514-{}.html?filter=lastpost", "page": 28},
            {"name": "穿越火线", "path": "https://club.hihonor.com/cn/forum-4515-{}.html?filter=lastpost", "page": 1},
            {"name": "QQ飞车手游", "path": "https://club.hihonor.com/cn/forum-4516-{}.html?filter=lastpost", "page": 1},
            {"name": "荣耀同城", "path": "https://club.hihonor.com/cn/forum-267-{}.html?filter=lastpost", "page": 2},
            {"name": "荣耀高校", "path": "https://club.hihonor.com/cn/forum-4346-{}.html?filter=lastpost", "page": 1},
            {"name": "荣耀创意精英挑战赛", "path": "https://club.hihonor.com/cn/forum-4528-{}.html?filter=lastpost", "page": 1},
            {"name": "大V申请", "path": "https://club.hihonor.com/cn/forum-150-{}.html?filter=lastpost", "page": 35},
            {"name": "申诉建议", "path": "https://club.hihonor.com/cn/forum-152-{}.html?filter=lastpost", "page": 3},
            {"name": "笔记本", "path": "https://club.hihonor.com/cn/forum-4333-{}.html?filter=lastpost", "page": 16},
            {"name": "平板", "path": "https://club.hihonor.com/cn/forum-4529-{}.html?filter=lastpost", "page": 22},
            {"name": "耳机音箱", "path": "https://club.hihonor.com/cn/forum-3719-{}.html?filter=lastpost", "page": 4}
        ]
        self.table_name = "huawei_bankuai_page"
        #User-Agent
        self.pc_useragent_list = [
            "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.3",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.110 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.110 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.186 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1",
            "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.79 Safari/537.36 Edge/14.14393",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.83 Safari/537.1",
            "Mozilla/5.0 (Windows NT 5.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
            "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36"
        ]

        #链接数据库
        try:
            self.db_225_connect = StoreMysqlPool(**config.db_225_test)
        except:
            self.db_225_connect = StoreMysqlPool(**config.db_225_test)
        self.store_queue = Queue()

    #请求页面
    def start_requests(self):
       sql = 'SELECT id,url FROM mum_url WHERE `status` = 0 limit 10;'
       data = self.db_225_connect.query(sql)
       for id,url  in data:
           self.id = id
           headers = {
               "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
               "User-Agent": random.choice(self.pc_useragent_list),
               "Cookie":"VONA_COMMON_LOG_KEY=8b16f212-f212-764e-470d-b0b5d5a2aed8; VONALOGID=881c783d432ddf2cd36f254f29bafa41; sajssdk_2015_cross_new_user=1; sensorsdata2015jssdkcross=%7B%22distinct_id%22%3A%2217ceeb79ca317-067dc9a3b6ad32-57b193e-2073600-17ceeb79ca4640%22%2C%22first_id%22%3A%22%22%2C%22props%22%3A%7B%22%24latest_traffic_source_type%22%3A%22%E7%9B%B4%E6%8E%A5%E6%B5%81%E9%87%8F%22%2C%22%24latest_search_keyword%22%3A%22%E6%9C%AA%E5%8F%96%E5%88%B0%E5%80%BC_%E7%9B%B4%E6%8E%A5%E6%89%93%E5%BC%80%22%2C%22%24latest_referrer%22%3A%22%22%7D%2C%22%24device_id%22%3A%2217ceeb79ca317-067dc9a3b6ad32-57b193e-2073600-17ceeb79ca4640%22%7D; krt.vis=92970e09-4014-4ef6-8c50-516b5cc659c6; Hm_lvt_c86e2c4bb4ad01427261d9484e89bf8b=1636092584; Hm_lpvt_c86e2c4bb4ad01427261d9484e89bf8b=1636092584; CUSTCD=; CustomerCode=; wechat_binded=; AUTHPHONE=; MISUMIVONAEC=%7b%22ABt%22%3a%22b%22%7d; VONAEC_TOP_WOS_BALOON=opened; depocity=%5b119%2c224%2c257%2c289%2c317%2c340%5d; Qs_lvt_294485=1636092584; Qs_pv_294485=2514124822837465600; _ga=GA1.3.2007463036.1636092584; _gid=GA1.3.216996045.1636092584; krt.context=session%3a84134014-1b92-4af6-b8cc-8e4096bb34fe%3bcontext_mode%3aother; _dc_gtm_UA-143948235-1=1; _ga-ch=GA1.3.2007463036.1636092584; _ga-ch_gid=GA1.3.506639854.1636092585; _dc_gtm_UA-6311415-1=1; _gaexp=GAX1.3.sG8KycnKQaK90ZtJ3XtaGQ.19014.0; _gat=1; mediav=%7b%22eid%22%3a%2267947%22%2c%22ep%22%3a%22%22%2c%22vid%22%3a%22%22%2c%22ctn%22%3a%22%22%2c%22vvid%22%3a%22%22%2c%22_mvnf%22%3a1%2c%22_mvctn%22%3a0%2c%22_mvck%22%3a1%2c%22_refnf%22%3a1%7d",
               "Host":"www.misumi.com.cn"
           }
           # requests.get(url=url,headers=headers,verify=False)
           spider_request = SpiderRequest(headers=headers, urls=url,config={"redirect": 1})
           self.sending_queue.put(spider_request)

           self.db_225_connect.do('UPDATE mum_url SET `status` = 1 WHERE id = {};'.format(self.id))
           time.sleep(1)
           break

    #详情返回
    def deal_response_results_status(self, task_status, url, result, request):
        if task_status == '2':
            ext_type = url["ext_type"]
            print("详情返回成功。。。")
            if ext_type==2:
                res = self.get_details_info(url, result['result'])
        else:
            print("下载中心方法失败")

    # 详情处理
    def get_details_info(self, url, html):
        print(self.id)
        print(html)
        pass


    #下载中心 用户账号、密码
    def get_user_password(self):
        return "test", "Welcome#1"
        # return 'caihaijun', 'caihaijun'

    def is_start(self):
        return self.sended_queue.qsize() < self.sended_queue_maxsize and self.sending_queue.qsize() < self.sended_queue_maxsize \
               and self.response_queue.qsize() < self.sended_queue_maxsize





if __name__ == "__main__":
    spider = mum_creat_tool(remote=True)
    timeout_time = -1  # 永不停止 18秒 停止
    process_pool = 5  # 线程数 debug用单线程
    spider.run(1, 1, 1, 1, timeout_time, timeout_time, timeout_time,
               timeout_time, True)









# -*- coding: utf8 -*-

from elasticsearch import Elasticsearch
import elasticsearch.helpers
from datetime import time


class StoreEs(object):
    def __init__(self, host="localhost", port=9200, params=None, hosts=[]):
        """
        连接集群 或者单个节点
        :param host:
        :param port:
        :param params:
        :param hosts:
        """
        if hosts:
            if params:
                self.es = Elasticsearch(hosts, **params)
            else:
                self.es = Elasticsearch(hosts)
        else:
            if params:
                self.es = Elasticsearch("{0}:{1}".format(host, port), **params)
            else:
                self.es = Elasticsearch("{0}:{1}".format(host, port))

    def save(self, index, doc_type, body, id=None, params=None):
        field = dict()
        field['index'] = index
        field['doc_type'] = doc_type
        field['body'] = body
        if id:
            field['id'] = id
        if params:
            field['params'] = params
        self.es.index(**field)

    def batch_save(self, index, doc_type, body):
        for i, val in enumerate(body):
            while True:
                try:
                    self.save(index, doc_type, val)
                    break
                    pass
                except:
                    time.sleep(1)
                    continue

    def bulk_by_ids(self, index, doc_type, body, parallel=False):
        """
        批量操作指定id
        :param index:
        :param doc_type:
        :param body:
        :return:
        """
        actions = [
            {
                '_op_type': 'index',
                '_index': index,
                '_type': doc_type,
                '_id': d.pop("id"),
                '_source': d
            }
            for d in body
            ]
        if parallel:
            for success, info in elasticsearch.helpers.parallel_bulk(client=self.es, actions=actions, thread_count=10,
                                                                     chunk_size=1000):
                if not success:
                    print('Doc failed', info)
        else:
            elasticsearch.helpers.bulk(client=self.es, actions=actions,
                                       stats_only=True)  # 如果为True，则仅报告成功/失败操作的数量，而不仅仅是成功的次数和错误响应列表

    def bulk(self, index, doc_type, body):
        actions = [
            {
                '_op_type': 'index',  # 该bulk()API接受index，create， delete，和update行动。使用该_op_type字段指定操作（_op_type默认为index）：
                '_index': index,
                '_type': doc_type,
                '_source': d
            }
            for d in body
            ]
        elasticsearch.helpers.bulk(client=self.es, actions=actions)

    def search(self, index, doc_type, body=None, params=None):
        if params is None:
            res = self.es.search(index=index, doc_type=doc_type, body=body)
        else:
            res = self.es.search(index=index, doc_type=doc_type, body=body, params=params)
        return res

    def get(self, index, doc_type, id):
        return self.es.get(index=index, doc_type=doc_type, id=id)

    def update_by_query(self, index, doc_type=None, body=None, params=None):
        return self.es.update_by_query(index=index, doc_type=doc_type, body=body, params=params)

    def delete(self, index, doc_type, id):
        self.es.delete(index=index, doc_type=doc_type, id=id)


if __name__ == '__main__':
    # import uuid
    # from Queue import Queue
    # from threading import Thread
    from download_center.store.store_es_v2 import StoreEs

    config = {
        'host': '115.159.158.157',
        'port': 9200,
        'params': {
            'http_auth': ('admin', 'Iknowthat221@')
        }
    }
    try:
        es = StoreEs(**config)
        result = es.get("content_center", "tiezi", "AWBaEjLkLUY1KU5jKSwu")
        print("")
    except:
        pass



# -*- coding: utf8 -*-
from collections import OrderedDict
from copy import deepcopy
import pymysql
import time
from DBUtils.PooledDB import PooledDB
from DBUtils.PersistentDB import PersistentDB
import traceback
from datetime import datetime
import warnings

warnings.filterwarnings('ignore')


class StoreMysqlPool(object):
    """
    mysql读写相关操作，pymysql，使用PooledDB连接池
    Args:
        host:数据库ip
        user:数据库用户名
        password:数据库用户密码
        db:数据库名
        port:数据库端口，默认3306
        mincached:最少的空闲连接数， 默认为1
        charset:数据库编码，默认utf8
    """

    def __init__(self, host="", user="", password="", db="", port=3306, mincached=1,
                 pattern=1, charset="utf8"):
        self.host = host

        if pattern == 1:
            self.pool = PooledDB(pymysql, mincached, host=host, user=user, passwd=password, db=db, port=port,
                                 charset=charset)
            self.close_able = True
        elif pattern == 2:
            self.pool = PersistentDB(pymysql, host=host, user=user, passwd=password, db=db, port=port,
                                     charset=charset)

            self.close_able = False
        self._last_use_time = time.time()

    def close(self, db):
        if db is not None:
            db.close()

    def connect(self):
        return self.pool.connection()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close_all()

    @staticmethod
    def _cursor(db):
        # return db.cursor(pymysql.cursors.DictCursor)
        return db.cursor()

    def query(self, sql):
        """
        根据sql查询
        Returns：
            数组的数组，外层数组元素为一行，内存数组元素为一行的一列
        """
        db = self.connect()
        cur = self._cursor(db)
        rows = []
        try:
            cur.execute(sql)
            db.commit()
            rows = cur.fetchall()
        except pymysql.OperationalError:
            print(traceback.format_exc())
        except Exception:
            print(traceback.format_exc())
        finally:
            cur.close()
            self.close(db)
        return rows

    def count(self, tb):
        """
        返回某表的行数
        Args:
            tb:字符串，表名称
        """
        sql = 'select count(*) from %s' % tb
        results = self.query(sql)
        if len(results) == 1 and len(results[0]) == 1:
            return int(results[0][0])

    def do(self, sql, flag='rowcount'):
        """
        执行sql，insert/delete/update操作
        Args:
            sql:要执行的sql
            flag:返回值类型，flag=lastrowid返回lastrowid，flag=rowcount返回rowcount
        """
        db = self.connect()
        cur = self._cursor(db)
        r = None
        try:
            cur.execute(sql)
            db.commit()
            r = 1
            if flag == 'lastrowid':
                r = cur.lastrowid
            elif flag == 'rowcount':
                r = cur.rowcount
        except pymysql.OperationalError:
            print("time: %s; Error on %s: %s" % (str(datetime.now()), self.host, sql[0: 200]))
            print(traceback.format_exc())
            r = -1
        except Exception:
            print("time: %s; Error to MySQL on %s: %s" % (str(datetime.now()), self.host, sql[0: 200]))
            print(traceback.format_exc())
            r = -1
        finally:
            cur.close()
            self.close(db)
        return r

    def save(self, table, data, mapping_fields=dict()):
        """
        将字典直接insert到数据库
        Args:
            table:字符串，插入目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            mapping_fields: 用于保持data字典的key与数据库字段的对应关系，
                            如果结果字典的某个key不包含在mapping_fields中，则将直接使用key作为字段名
        """

        if len(data) <= 0:
            return -1
        try:
            fields = ''
            values = ''
            for d in data:
                if d in mapping_fields:
                    fields += "`%s`," % (str(mapping_fields[d]))
                else:
                    fields += "`%s`," % (str(d))
                values += "'%s'," % (pymysql.escape_string(str(data[d])))
            if len(fields) <= 0 or len(values) <= 0:
                return -1
            sql = "insert ignore into %s(%s) values(%s)" % (table, fields[:-1], values[:-1])
            return self.do(sql)
        except Exception:
            print(traceback.format_exc())
            return -1

    def saveMany(self, sql, values, flag='lastrowid'):
        """
        执行sql，insert/delete/update操作
        :param table: 'insert into table(id,name) values(%s,%s)'
        :param names: 字符串
        :param values:[{1,2},{1,2},...]  是一个列表，列表中的每一个元素必须是元组！！！
        :param flag:
        :return:
        """
        db = self.connect()
        cur = self._cursor(db)
        r = None
        try:
            cur.executemany(sql, values)
            db.commit()
            r = 1
            if flag == 'lastrowid':
                r = cur.lastrowid
            elif flag == 'rowcount':
                r = cur.rowcount

        except pymysql.OperationalError:
            print("Error connecting to MySQL on %s: %s" % (self.host, sql[0:255]))
            print(traceback.format_exc())
            r = -1
        except Exception:
            print("Error connecting to MySQL on %s: %s" % (self.host, sql[0:255]))
            print(traceback.format_exc())
            r = -1
        finally:
            cur.close()
            self.close(db)
        return r

    def update(self, table, data, field, mapping_fields=dict()):
        """
        将字典直接update到数据库
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据该字段判断是否为同一条记录，作为where条件
            mapping_fields: 用于保持data字典的key与数据库字段的对应关系，
                            如果结果字典的某个key不包含在mapping_fields中，则将直接使用key作为字段名
        """
        if len(data) <= 0:
            return -1
        else:
            try:
                values = ''
                field_value = None
                for d in data:
                    key = d
                    if d in mapping_fields:
                        key = mapping_fields[d]
                    if key == field:
                        field_value = data[d]
                    values += "%s='%s'," % (str(key), pymysql.escape_string(str(data[d])))
                if len(values) <= 0 or field_value is None:
                    return -1
                sql = "update " + table + " set " + values[:-1] + " where " + field + "='" + pymysql.escape_string(
                        str(field_value)) + "'"
                return self.do(sql, flag='rowcount')
            except Exception:
                print(traceback.format_exc())
                return -1

    def saveorupdate(self, table, data, field, mapping_fields=dict()):
        """
        将字典更新到数据库，如果已存在则update，不存在则insert
        Args:
            table:字符串，更新目标表的名称
            data:字典格式，key为字段名称，value为字段值，如{'id':'1','name':'temp'}
            field:唯一索引字段，即根据词字段判断是否为同一条记录，作为where条件
            mapping_fields: 用于保持data字典的key与数据库字段的对应关系，
                            如果结果字典的某个key不包含在mapping_fields中，则将直接使用key作为字段名
        """
        if len(data) <= 0:
            return -1
        try:
            field_value = None
            if field in data:
                field_value = data[field]
            else:
                for key in mapping_fields:
                    if mapping_fields[key] == field and key in data:
                        field_value = data[key]
            if field_value is None:
                return -1
            querysql = "select count(1) from " + table + " where " + field + "='" + pymysql.escape_string(
                    str(field_value)) + "'"
            ed = self.query(querysql)
            if ed and ed[0][0] > 0:
                return self.update(table, data, field, mapping_fields)
            else:
                return self.save(table, data, mapping_fields)
        except Exception:
            print(traceback.format_exc())
            return -1

    def tableCreate(self, tb_name, likeTable):
        """
        创建数据表格
        :param tb_name: table's name
        :return: 1 ok , -1 error
        """
        # sql = "SELECT table_name FROM information_schema.TABLES WHERE table_name ='{}';".format(tb_name)
        sql = "show tables;"
        datas = self.query(sql)
        if tb_name in [str(i[0]) for i in datas]:
            # 存在
            sql = "select * from {} limit 10;".format(tb_name)
            if not self.query(sql):
                print("{} 是空表, 正在truncate.....".format(tb_name))
                sql = " truncate {};".format(tb_name)
                datas = self.query(sql)
            else:
                print("表格存在：{}".format(tb_name))

        else:
            # 不存在 创建 清空
            print("{} 表格不存在，正在创建....".format(tb_name))
            sql = "create table {} like {};".format(tb_name, likeTable)
            datas = self.query(sql)
            sql = " truncate {};".format(tb_name)
            datas = self.query(sql)
        return 1

    def dictsToOrderDicts(self, list_dicts):
        alist = []
        # list_dicts = [list_dicts] if isinstance(list_dicts, dict) else list_dicts
        ks = list_dicts[0].keys()  # 有序
        order_dict = OrderedDict()
        for adict in list_dicts:
            for k in ks:
                order_dict[k] = adict[k]
            alist.append(deepcopy(order_dict))
        return alist

    def insert_many(self, table, list_dicts, conflict=''):
        '''
        [批量插入]
        :param store_name:
        :param table:
        :param list_dicts:
        :param conflict: 冲突更新字段 local_date=CURRENT_DATE() or ['local_date','fields']
        :return:
        '''
        try:
            if len(list_dicts):
                orderDicts = self.dictsToOrderDicts(list_dicts)
                ks = orderDicts[0].keys()  # 获取键值
                if conflict:
                    if isinstance(conflict, str):
                        insert_sql = "insert ignore into {} ({}) values ({})  ON DUPLICATE KEY UPDATE {}".format(
                                table, ", ".join(ks), ("%s," * len(ks)).strip(","), conflict)
                    else:
                        if len(conflict):
                            conflict = ','.join(['{}=VALUES({})'.format(field, field) for field in conflict])
                            insert_sql = "insert ignore into {} ({}) values ({})  ON DUPLICATE KEY UPDATE {}".format(
                                    table, ", ".join(ks), ("%s," * len(ks)).strip(","), conflict)
                        else:
                            return -1

                else:
                    insert_sql = "insert ignore into {} ({}) values ({})".format(table, ", ".join(ks),
                                                                                 ("%s," * len(ks)).strip(","))
                values = []
                for adict in list_dicts:
                    values.append(tuple(adict.values()))
                res = self.saveMany(insert_sql, values, flag="rowcount")
                return res
            else:
                return -3
        except:
            return -2

    def close_all(self):
        """
        关闭连接池全部连接
        请在确保连接不再需要时确认
        :return:
        """
        if self.close_able:
            self.pool.close()


def test():
    db = {
        'host': 'localhost',
        'user': 'root',
        'password': '',
        'db': 'test'
    }
    mq = StoreMysqlPool(**db)
    print(mq.count('urls'))


if __name__ == "__main__":
    test()



# -*- coding: utf8 -*-

import traceback
import pymysql

from pymysql.converters import escape_string
from download_center.store.py_store_mysql_pool import StoreMysqlPool


class SourceStore(object):

    def __init__(self, db_conn):
        # 数据库连接信息
        self.db = StoreMysqlPool(**db_conn)

    def store_table(self, results, table="", type=1, field=None):
        try:
            if len(results) > 0:
                for result in results:
                    if type == 1:
                        for key in result:
                            result[key] = escape_string(str(result[key]))
                        return_state = self.db.save(table, result)
                    elif type == 2:
                        for key in result:
                            result[key] = escape_string(str(result[key]))
                        return_state = self.db.update(table, result, field)
                return return_state
        except Exception:
            print(traceback.format_exc())
            return -1

    def store_table_one(self, result, table="", type=1, field=None):
        """
        单个  1 保存或 2 更新
        :param result:
        :param table:
        :param type: 1
        :param field:
        :return:
        """
        try:
            if type == 1:
                for key in result:
                    result[key] = escape_string(str(result[key]))
                return_state = self.db.save(table, result)
            elif type == 2:
                for key in result:
                    result[key] = escape_string(str(result[key]))
                return_state = self.db.update(table, result, field)
            return return_state
        except Exception:
            print(traceback.format_exc())
            return -1

    def store_table_db(self, results, table="", type=1, field=None):
        return_state = 0
        if len(results) > 0:
            for result in results:
                try:
                    if type == 1:
                        for key in result:
                            result[key] = escape_string(str(result[key]))
                        return_state = self.db.save(table, result)
                    elif type == 2:
                        for key in result:
                            result[key] = escape_string(str(result[key]))
                        return_state = self.db.update(table, result, field)
                except:
                    print(traceback.format_exc())
                    return -1
            return return_state

    def saveorupdate(self, results, table="", field=None):
        try:
            if len(results) > 0:
                for result in results:
                    for key in result:
                        result[key] = escape_string(str(result[key]))
                    return_state = self.db.saveorupdate(table, result, field)
                return return_state
        except Exception:
            print(traceback.format_exc())
            return -1

    def store_insert_or_update(self, results, table="", field=None, isupdate=1):
        """
        isupdate == 1 更新  2： pass  不传field 直接保存  saveorupdate
        :param results:
        :param table:
        :param field:
        :param isupdate:
        :return:
        """
        try:
            store_id = 0
            if len(results) > 0:
                for result in results:
                    # values = ''
                    field_value = None
                    for key in result:
                        result[key] = escape_string(str(result[key]))
                        if key == field:
                            field_value = result[key]
                        # values += "%s='%s'," % (str(key), str(result[key]))
                    if field:
                        field_result = self.find_by_field(table, field, field_value)
                        if field_result:
                            if isupdate == 1:
                                self.db.update(table, result, field)
                            else:
                                pass
                        else:
                            store_id = self.db.save(table, result)
                    else:
                        store_id = self.db.save(table, result)
                return store_id
        except Exception:
            print(traceback.format_exc())
            return -1

    def find_by_field(self, table_name, field, field_value):
        try:
            sql = "select * from %s where %s = '%s' " % (table_name, field, field_value)
            result = self.db.query(sql)
            return result
        except Exception:
            print(traceback.format_exc())

    def store_update(self, result, ty, field):
        i = 0
        for key in result:
            if result[key] and key != field:
                result[key] = escape_string(str(result[key]))
                i += 1
        if i > 0:
            self.db.update(ty, result, field)

    def store_insert(self, result, ty):
        for key in result:
            result[key] = escape_string(str(result[key]))
        self.db.save(ty, result)

    def deleteByids(self, ids, table=""):
        for single_id in ids:
            try:
                sql = "delete from %s  where  id = %d " % (table, single_id['id'])
                self.db.query(sql)
            except Exception:
                print(traceback.format_exc())


def test():
    import pymysql
    # print(escape_string("fksdfsfkj粉红色开发商可‘是方法'kfsdk'sdhgfskd"))

    config_db = {
        'host': '182.254.244.167',
        # 'host': '10.131.221.0',
        'user': 'sx',
        'password': 'sx@0809',
        'db': 'test_rank'
    }
    db_pool = SourceStore(config_db)
    table = "test"

    data_list = [{"keywprd": "11"}, {"keywprd": "33"}, {"keywprd": "44"}]
    insert_sql = "insert ignore into {}(keyword) values".format(table)
    for r in data_list:
        insert_sql += "('{}') ,".format(r["keywprd"])
    print(insert_sql[0:-1])
    db_pool.db.do(insert_sql[0:-1])

if __name__ == '__main__':
    test()
